/*************************************************************30.05.2008*/
/* class MthException: error management					*/
/************************************************************************/

#ifndef MTHEXCEPTION_H
#define MTHEXCEPTION_H


class MthException {
  public:
    enum mth_error {
      failure,				// generic failure
      convergence,			// no convergence
      dimensions,			// incompatible dimensions
      square,				// matrix not square
      singular,				// matrix singular
      duplicate,			// duplicate x values
      ascending,			// x values not in ascending order
      n_data,				// too few data points
      option,				// option not implemented
      increment,			// increment too small, wrong sign
      range,				// value out of range
      interval,				// interval too narrow or ill-defined
      ch_sign,				// no change of sign
      minimum,				// no local minimum
      inflection,			// inflection pt. in step function
      collinear,			// collinear points
      irrelevant,			// parameter irrelevant
      initialization,			// variable not initialized
      data_loss				// data loss (demotion of a variable)
    };

    MthException(mth_error c): code(c), k(-1) {}
    MthException(mth_error c, int i): code(c), k(i) {}
    const char* message() const noexcept {
      static char buffer[80];
      if (k < 0)
	return errmsg[code];
      else {
        sprintf(buffer, "%s: %d", errmsg[code], k);
	return buffer;
      }
    }
    int number() const noexcept {
      return code;
    }

  private:
    mth_error code;
    int k;
    static const char *errmsg[];
};

#ifdef MAIN
const char *MthException::errmsg[] = {
  "generic failure",
  "too many iteration cycles",
  "incompatible matrix/vector dimensions",
  "matrix not square",
  "matrix singular",
  "duplicate x data",
  "abscissa values not in ascending order",
  "too few data points/equations",
  "option/approximation level not implemented",
  "increment/initial step size too small or wrong sign",
  "value out of range",
  "interval too narrow, or reversed boundaries",
  "initial y values have the same signs",
  "initial values do not show a local minimum",
  "inflection point in step function",
  "points collinear",
  "parameter irrelevant #"
  "uninitialized variable"
  "data loss"
}; 
#endif
#endif
