/************************************************************************/
/* some functions to be implemented as inline functions for better per-	*/
/* formance								*/
/************************************************************************/

#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#ifndef SIMPLE_H
#include <simple.h>	// need sq()
#endif

/*************************************************************13.12.2015*/
/* f_erfN:  error function						*/
/* f_erfcN: complementary error function				*/
/* These are fast implementations for positive arguments only		*/
/* and with lower precision (max. deviation of version 1: 5.0e-4,	*/
/* version 2: 3.0e-7; use erf() and erfc() from	the C library if	*/
/* machine precision is required)					*/
/*									*/
/* [1] M. Abramowitz and I. Stegun, Handbook of Mathematical Functions,	*/
/*     with Formulas, Graphs, and Mathematical Tables, Dover Public-	*/
/*     ations, 1964.							*/
/* [2] https://en.wikipedia.org/wiki/Error_function			*/
/************************************************************************/
inline double f_erfc1(double x) noexcept {
  const double a[5] = {1.0, 0.278393, 0.230389, 0.000972, 0.078108};
  return sq(sq(1.0 / (a[0] + x * (a[1] + x * (a[2] + x * (a[3]
  + x * a[4]))))));
}

inline double f_erf1(double x) noexcept {
  return 1.0 - f_erfc1(x);
}

inline double f_erfc2(double x) noexcept {
  const double a[7] = {1.0, 0.0705230784, 0.0422820123, 0.0092705272,
    0.0001520143, 0.0002765672, 0.0000430638};
  return sq(sq(sq(sq(1.0 / (a[0] + x * (a[1] + x * (a[2] + x * (a[3]
  + x * (a[4] + x * (a[5] + x * a[6]))))))))));
}

inline double f_erf2(double x) noexcept {
  return 1.0 - f_erfc2(x);
}
#endif
