/************************************************************************/
/* simple.h: some useful simple template functions			*/
/************************************************************************/

#ifndef SIMPLE_H
#define SIMPLE_H


// inline functions

template <typename T>			// sign function
inline int signum(const T& x) {
  return (T(0) < x) - (x < T(0));
}

template <typename T>			// square of a number
inline T sq(const T& x) {
  return x * x;
}

template <typename T>			// cube of a number
inline T cb(const T& x) {
  return x * x * x;
}

template <typename T>			// 4th power
inline T pwr4(const T& x) {
  return sq(sq(x));
}

template <typename T>			// arbitrary integer power
T powi(const T& x, int n) {
  if (n < 0)
    return powi(static_cast<T>(1) / x, -n);
  else if (n == 0)
    return static_cast<T>(1);		// x^0 = 1 even for x == 0
  else {
    T y(x), xpwr(x);
    n--;
    while (n > 0) {
      if (n % 2 == 1) {
	y *= xpwr;
	n--;
      }
      xpwr = sq(xpwr);
      n /= 2;
    }
    return y;
  }
}
    
#if 0	// functions in standard library (utility)
template <typename T>			// maximum value
  inline T max(const T& x, const T& y) {
  return ((x > y)? x : y);
}
#ifdef XCPP
template <typename T, typename... Args>
inline T max(const T& x, const Args&... other) {
  return max(x, max(other...));
}
#endif

template <typename T>			// minimum value
inline T min(const T& x, const T& y) {
  return ((x < y)? x : y);
}
#ifdef XCPP
template <typename T, typename... Args>
inline T min(const T& x, const Args&... other) {
  return min(x, min(other...));
}
#endif


// other functions
template <typename T>
void swap(T& x, T& y);

template <typename T>			// swap 2 simple objects
void swap(T& x, T& y) {
  const T tmp = x;
  x = y;
  y = tmp;
}
#endif	// functions in std library
#endif
