/*************************************************************03.09.2001*/
/* eqn_quadratic: real roots of a quadratic equation			*/
/*									*/
/* Equations of lesser degree are solved by the appropriate formulas.	*/
/* The solutions are arranged in ascending order.			*/
/*======================================================================*/
/* a		(i)  vector containing the polynomial coefficients
   x		(o)  result array
   RESULT	(o)  number of valid solutions				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


int eqn_quadratic(const Polynomial<double>& a, Vector<double>& x) noexcept {
  int nreal;
  double p, dis;

  if (a.size() >= 3 && a[2] != 0.0) {	// quadratic problem?
    p = 0.5 * a[1] / a[2];
    dis = sq(p) - a[0] / a[2];
    if (dis >= 0.0) {			// two real solutions
      dis = sqrt(dis);
      x[0] = -p - dis;
      x[1] = -p + dis;
      nreal = 2;
    }
    else				// no real solution
      nreal = 0;
  }
  else if (a.size() >= 2 && a[1] != 0.0) {// linear problem?
    x[0] = -a[0] / a[1];
    nreal = 1;
  }
  else					// no equation
    nreal = 0;

  return nreal;
}
