/*************************************************************28.01.2017*/
/* eqn_seek: vary the argument of a 1-dimensional monotonic function	*/
/*	     until it returns a non-negative value			*/
/*======================================================================*/
/* crit(x, u)	(i)  function defining the criterion (search stops when
		     crit() >= 0.0
   x		(io) initial/final argument value
   u		(i)  struct containing data required by crit()
   step		(i)  factor by which x is in/decremented
   RESULT	(o)  final function value
   EXCEPTIONS	     failure: function not monotonic
   		     range: step size not positive
   		     exceptions thrown by crit()			*/
/************************************************************************/
#include <mathC.h>

using namespace std;


double eqn_seek(function<double(double, void *)> crit, void *u, double& x,
double step) {
  bool forward;
  double f, f_old;

  if (step <= 0.0) throw MthException(MthException::range);
  if ((f_old = crit(x, u)) >= 0.0) return f_old;
  x *= step;
  if ((f = crit(x, u)) >= 0.0) return f;
  if (forward = (f >= f_old))		// move in direction of "step"
    f_old = f;
  else					// ... or use the other direction
    x /= step;
  do {
    x *= (forward) ? step : 1.0 / step;
    if ((f = crit(x, u)) >= 0.0) return f;
    if (f < f_old) throw MthException(MthException::failure);
    f_old = f;
  } while (true);
}
