/************************************************************************/
/* root of a 1-dimensional function using Steffensen's accelerated	*/
/* substitution algorithm						*/
/************************************************************************/
#include <mathC.h>

using namespace std;


/*************************************************************12.12.2001*/
/* eqn_steffensen: 1-dimensional root finder using Steffensen's		*/
/*		   accelerated substitution algorithm			*/
/*======================================================================*/
/* f(x, u)	(i)  iteration step function (solution x = f(x, u))
   u		(i)  structure for passing additional information to f()
   x		(i)  initial value
   kmax		(i)  number of iteration steps
   eps		(i)  convergence criterion (max. x deviation)
   RESULT	(o)  x value
   exceptions	     inflection:  algorithm failure (inflection point)
		     convergence: no convergence after kmax steps
		     exceptions thrown by f()				*/
/************************************************************************/
double eqn_steffensen(function<double(double, void *)> f, void *u,
double x, int kmax, double eps) {
  int i;
  double x0, x1, x2, d, ds;

  for (i = 0; i < kmax; i++) {
    x0 = x;
    x1 = f(x0, u);
    if (fabs(x1 - x0) < mth_eps0 * fabs(x0)) return x;
    x2 = f(x1, u);
    d = (x2 - x1) - (x1 - x0);
    if (d == 0.0) throw MthException(MthException::inflection);
    ds = sq(x1 - x0) / d;
    x = x0 - ds;
    if (fabs(ds) < eps) return x;
  }
  throw MthException(MthException::convergence);
}
