/************************************************************************/
/* Gamma function, factorial, etc.					*/
/************************************************************************/

#include <mathC.h>

using namespace std;


#if 0	// function obsolete, use tgamma() or lgamma() from the C library
/*************************************************************25.08.2005*/
/* f_gammaln: logarithm of Gamma function				*/
/*									*/
/* after: Numerical Recipes						*/
/*======================================================================*/
/* x		(i)  argument (must be positive)
   RESULT	(o)  ln(Gamma(x))
   exceptions	     range: x < 0.0					*/
/************************************************************************/
double f_gammaln(double x) {
  double y, tmp, ser;
  static double coeff[6] = {76.18009172947146, -86.50532032941677,
    24.01409824083091, -1.231739572450155, 0.1208650973866179E-2,
    -0.5395239384953E-5};
  int j;

  if (x < 0.0) throw MthException(MthException::range);
  y = x;
  tmp = x + 5.5;
  tmp -= (x + 0.5) * log(tmp);
  ser = 1.000000000190015;
  for (j = 0; j <= 5; j++) ser += coeff[j] / ++y;
  return -tmp + log(2.5066282746310005 * ser / x);
}
#endif

/*************************************************************25.08.2005*/
/* f_dfac: factorial (as floating point)				*/
/*									*/
/* after: Numerical Recipes						*/
/*======================================================================*/
/* n		(i)  argument (0 <= n <= 170, upper limit valid for
		     IEEE 754 double precision arithmetics)
   RESULT	(o)  n!
   exceptions	     range: n < 0, n > 170				*/
/************************************************************************/
double f_dfac(int n) {
  static int nfill = 6;
  static double a[171] = {1.0, 1.0, 2.0, 6.0, 24.0, 120.0, 720.0};

  if (n < 0 || n > 170) throw MthException(MthException::range);

  while (nfill < n) {
    nfill++;
    a[nfill] = a[nfill - 1] * nfill;
  }
  return a[n];
}

/*************************************************************25.08.2005*/
/* f_ibinom_c: binomial coefficents (as integers)			*/
/*									*/
/* method: Pascal's triangle, results are remembered			*/
/*======================================================================*/
/* n, i		(i)  arguments
   RESULT	(o)  (n i) ("n over i")
   exceptions	     range: n too high					*/
/************************************************************************/
const int nbcmax = 40;			// largest n value allowed


int f_ibinom_c(int n, int i) {
  int j, *p1, *p2;
  static int nfill = -1, bc[(nbcmax + 1) * (nbcmax + 2) / 2];

  if (n < 0 || n > nbcmax) throw MthException(MthException::range);
  if (i < 0 || i > n) return 0;

  if (nfill < 0) {			// initialize
    bc[0] = 1;
    nfill = 0;
  }

  if (n > nfill) {			// add rows to table
    p1 = bc + nfill * (nfill + 1) / 2;	// begin of last filled row
    p2 = p1 + nfill + 1;		// begin of 1st empty row
    while (nfill < n) {
      nfill++;
      *p2++ = 1;
      for (j = 1; j < nfill; j++, p1++, p2++) *p2 = *p1 + *(p1 + 1);
      *p2++ = 1;
      p1++;
    }
  }

  return bc[n * (n + 1) / 2 + i];
}
