/************************************************************************/
/* Legendre polynomials							*/
/************************************************************************/

#include <mathC.h>

using namespace std;


/*************************************************************24.08.2005*/
/* f_legendre: Legendre polynomial P_{l}(x)				*/
/*									*/
/* Legendre polynomials are orthogonal functions over [-1, +1]. 	*/
/* This function calculates the value of the polynomial. See		*/
/* f_legendre_c() if the coefficients are needed.			*/
/*									*/
/* note: Int (P_l(x))^2 dx = 2/(2l + 1)					*/
/*======================================================================*/
/* x		(i)  distance
   l		(i)  order (l >= 0)
   RESULT	(o)  P_l(x)
   exceptions	     option: l negative					*/
/************************************************************************/
double f_legendre(double x, int l) {
  int k;
  double p0, p1, p2;

  if (l < 0) throw MthException(MthException::option);
  p1 = 1.0;
  if (l == 0) return p1;

  p2 = x;
  if (l == 1) return p2;
  for (k = 2; k <= l; k++) {
    p0 = p1;
    p1 = p2;
    p2 = ((2 * k - 1) * x * p1 - (k - 1) * p0) / ((double) k);
  }
  return p2;
}

/*************************************************************31.05.2008*/
/* f_legendre_c: Legendre polynomial P_{l}(x)				*/
/*									*/
/* This functions calculates the coefficents of Legendre polynomials.	*/
/* See f_legendre(), if only the function value is needed.		*/
/*======================================================================*/
/* l		(i)  order (l >= 0)
   RESULT	(o)  vector of coefficients (must have length l + 1)
   exceptions	     option: l negative					*/
/************************************************************************/
Polynomial<double> f_legendre_c(int l) {
  int i, k;
  if (l < 0) throw MthException(MthException::option);
  Polynomial<double> p0(l + 1), p1(l + 1), c(l + 1);
  c = 0;
  c[0] = 1.0;				// P_0(x) = 1
  if (l == 0) return c;

  p1 = c;
  c[0] = 0.0;
  c[1] = 1.0;				// P_1(x) = x
  if (l == 1) return c;

  for (k = 2; k <= l; k++) {
    p0 = p1;
    p1 = c;
    c[0] = -((double) (k - 1)) / ((double) k) * p0[0];
    for (i = 1; i <= k; i ++) c[i] = ((2 * k - 1) * p1[i - 1]
    - (k - 1) * p0[i]) / ((double) k);
  }
  return c;
}
