/*************************************************************30.12.2017*/
/* int_ptrapez: integration of a periodic function by means of the	*/
/*		trapezoidal method (automatic number of nodes)		*/
/*======================================================================*/
/* f(x, u)	(i)  integrand function (x: scalar); if f() is not periodic
		     with period length a, you get what you deserve!
   u		(i)  pointer to additional data needed by f()
   a		(i)  period length of f()
   eps		(i)  desired precision (> 0: absolute, < 0: relative)
   RESULT	(o)  integral value
   EXCEPTIONS	     convergence: eps criterion not fulfilled		*/
/************************************************************************/
#include <mathC.h>

using namespace std;


double int_ptrapez(function<double(double, void *)> f, void *u, double a,
double eps) {
  int i, n;
  const int nmax = 32768;
  double h, s1, s2, smax,
    thr = eps;

  n = 4;				// (initial) number of nodes
  h = a / n;
  for (i = 0, s1 = 0.0; i < n; i++) s1 += f(i * h, u);
  s1 *= h;				// 1st approximation
  smax = fabs(s1);

  do {
    n *= 2;
    h *= 0.5;
    for (i = 1, s2 = 0.0; i < n; i += 2) s2 += f(i * h, u);
    s2 = (0.5 * s1 + s2 * h);
    smax = fmax(smax,fabs(s2));
    if (eps < 0.0) thr = fabs(eps * smax);
    if (fabs(s2 - s1) < thr) return s2;
    s1 = s2;
  } while (n <= nmax);
  throw MthException(MthException::convergence);
}

double int_ptrapez(function<double(double)> f, double a, double eps) {
  return int_ptrapez([&](double x, void *u) -> double {
    return f(x);
  }, NULL, a, eps);
}
