/*************************************************************16.08.2012*/
/* int_spline: integrate a cubic spline function			*/
/*======================================================================*/
/* x, y		(i)  abscissa and ordinate vectors as used by ipl_spline()
   c		(i)  coefficient matrix generated by ipl_spline()
   a, b		(i)  integration range
   RESULT	(o)  value of spline function				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


double int_spline(const SortedVector<double>& x, const Vector<double>& y,
const Matrix<double>& c, double a, double b) noexcept {
  int i, ia, ib, j,
    n = x.size();
  double h, q;

  if (b >= a)
    j = 1;
  else {
    j = -1;
    swap(a, b);
  }

  ia = x.floor_idx(a);
  ib = x.floor_idx(b);

  if (ia < 0) {			      // first (incomplete) interval
    h = a - x[0];
    q = -(0.5 * c[0][0] * h + y[0]) * h;
  }
  else if (ia < n - 1) {
    h = a - x[ia];
    q = -(((0.25 * c[2][ia] * h + 1.0 / 3.0 * c[1][ia]) * h
    + 0.5 * c[0][ia]) * h + y[ia]) * h;
  }
  else {
    h = a - x[ia];
    q = -(0.5 * c[0][ia] * h + y[ia]) * h;
  }

  for (i = max(ia, 0); i < min(ib, n - 1); i++) {
    h = x[i + 1] - x[i];	      // full intervals
    q += (((0.25 * c[2][i] * h + 1.0 / 3.0 * c[1][i]) * h
    + 0.5 * c[0][i]) * h + y[i]) * h;
  }

  if (ib < 0) {		             // last (incomplete) interval
    h = b - x[0];
    q += (0.5 * c[0][0] * h + y[0]) * h;
  }
  else if (ib < n - 1) {
    h = b - x[ib];
    q += (((0.25 * c[2][ib] * h + 1.0 / 3.0 * c[1][ib]) * h
    + 0.5 * c[0][ib]) * h + y[ib]) * h;
  }
  else {
    h = b - x[ib];
    q += (0.5 * c[0][ib] * h + y[ib]) * h;
  }

  return j * q;
}
