/*************************************************************16.12.2001*/
/* ipl_eval: evaluation of the spline function defined by		*/
/*	     subroutines ipl_spline or ipl_akima			*/
/*======================================================================*/
/* xvalue	(i)  abscissa value
   x, y		(i)  abscissa and ordinate vectors as used by ipl_spline()
   c		(i)  coefficient matrix generated by ipl_spline()
   k		(i)  order of derivative (0 <= k <= 3)
   RESULT	(o)  value of spline function				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


double ipl_eval(double xvalue, const SortedVector<double>& x,
const Vector<double>& y, const Matrix<double>& c, int k) noexcept {
  int i,
    n = x.size();
  double h, yvalue;

  i = x.floor_idx(xvalue);
  if (i < 0) {				// xvalue below x[0]
    switch (k) {
    case 0:
      yvalue = y[0] + c[0][0] * (xvalue - x[0]);
      break;
    case 1:
      yvalue = c[0][0];
      break;
    default:
      yvalue = 0.0;
    }
  }
  else if (i < n - 1) {			// x[0] <= xvalue < x[n-1]
    h = xvalue - x[i];
    switch (k) {
    case 0:
      yvalue = ((c[2][i] * h + c[1][i]) * h + c[0][i]) * h + y[i];
      break;
    case 1:
      yvalue = (3 * c[2][i] * h + 2 * c[1][i]) * h + c[0][i];
      break;
    case 2:
      yvalue = 6 * c[2][i] * h + 2 * c[1][i];
      break;
    case 3:
      yvalue = 6 * c[2][i];
      break;
    default:
      yvalue = 0.0;
    }
  }
  else {				// x[0] >= x[n-1]
    switch (k) {
    case 0:
      yvalue = y[n - 1] + c[0][n - 1] * (xvalue - x[n - 1]);
      break;
    case 1:
      yvalue = c[0][n - 1];
      break;
    default:
      yvalue = 0.0;
    }
  }
  return yvalue;
}
