/*************************************************************02.01.2003*/
/* ipl_lagrange: Lagrange interpolation polynomial			*/
/*									*/
/* G. Jordan-Engeln, F. Reutter: Numerische Mathematik für Ingenieure,	*/
/* BI Hochschultaschenbücher Bd. 104, Bibliographisches Institut,	*/
/* Mannheim 1973, pp. 198ff						*/
/*======================================================================*/
/* x, y		(i)  vectors of abscissa and ordinate values
   RESULT	(o)  vector of coefficients of Lagrange polynomial
   exceptions	     duplicate:   some x values are identical
		     dimensions:  x and y vector have different sizes	*/
/************************************************************************/
#include <mathC.h>

using namespace std;


Polynomial<double> ipl_lagrange(const Vector<double>& x,
const Vector<double>& y) {
  int i, j, k, s,
    n = x.size();
  double f;
  Polynomial<double> q(n), a(n);

  if (n != y.size()) throw MthException(MthException::dimensions);

  a.set(0.0);

  for (k = 0; k < n; k++) {
    q.set(0.0);
    q[0] = 1.0;
    for (j = s = 0, f = 1.0; j < n; j++) {
      if (j == k) continue;
      s++;
      for (i = s; i >= 1; i--) q[i] = -x[j] * q[i] + q[i - 1]; 
      q[0] *= -x[j];
      f *= (x[k] - x[j]);
    }
    if (f == 0.0) throw MthException(MthException::duplicate);
    a += (y[k] / f) * q;
  }
  return a;
}
