/*************************************************************17.04.2003*/
/* ipl_peval: evaluation of the periodic spline function defined by	*/
/*	      ipl_pspline						*/
/*======================================================================*/
/* xvalue	(i)  abszissa value
   x, y		(i)  vectors of abszissa and ordinate data
   c		(i)  coefficient matrix generated by ipl_spline()
   k		(i)  order of derivative (0 <= k <= 3)
   RESULT	(o)  value of spline function				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


double ipl_peval(double xvalue, const SortedVector<double>& x,
const Vector<double>& y, const Matrix<double>& c, int k) noexcept {
  int i,
    n = x.size();
  double h, yvalue, period;

  if (xvalue < x[0] || xvalue > x[n - 1]) {
    period = x[n - 1] - x[0];
    i = static_cast<int>((xvalue - x[0]) / period);
    xvalue -= i * period;
  }

  i = x.floor_idx(xvalue);
  h = xvalue - x[i];
  switch (k) {
    case 0:
      yvalue = ((c[2][i] * h + c[1][i]) * h + c[0][i]) * h + y[i];
      break;
    case 1:
      yvalue = (3 * c[2][i] * h + 2 * c[1][i]) * h + c[0][i];
      break;
    case 2:
      yvalue = 6 * c[2][i] * h + 2 * c[1][i];
      break;
    case 3:
      yvalue = 6 * c[2][i];
      break;
    default:
      yvalue = 0.0;
  }
  return yvalue;
}
