/*************************************************************24.08.2017*/
/* lin_eigen2: eigenvalues and -vectors of a symmetric matrix with 1 or	*/
/*	       2 rows							*/
/*======================================================================*/
/* a		(i)  symmetric matrix for which eigenvalues are sought
		     NOTE: We do not check for symmetry. The subdiagonal
		     elements are ignored.
   d		(o)  vector of eigenvalues, sorted (ascending order)
   v		(o)  matrix of which each row (v[i]), is an eigenvector
   exceptions	     square:      not a square matrix			*/
/************************************************************************/
#include <mathC.h>

using namespace std;


void lin_eigen2(const Matrix<double>& a, Vector<double>& d,
Matrix<double>& v) {
  int i,
    n = a.rows();
  double h;

  if (n != a.cols()) throw MthException(MthException::square);
  if (n > 2) throw MthException(MthException::dimensions);

  d.size(n);
  v.size(n, n);
  if (n == 1) {
    d[0] = a[0][0];
    v[0][0] = 1.0;
  }
  else {
    h = sqrt(0.25 * sq(a[0][0] - a[1][1]) + sq(a[0][1]));
    d[0] = 0.5 * (a[0][0] + a[1][1]) - h;
    d[1] = 0.5 * (a[0][0] + a[1][1]) + h;
    if ( h == 0.0) {			// eigenvector, degenerate case
      v[0][0] = v[1][1] = 1.0;
      v[0][1] = v[1][0] = 0.0;
    }
    else {				// ... normal case
      for (i = 0; i < n; i++) {
	if (fabs(a[0][0] - d[i]) < fabs(a[0][1])) {
	  v[i][0] = 1.0;
	  v[i][1] = -(a[0][0] - d[i]) / a[0][1];
	}
	else {
	  v[i][1] = 1.0;
	  v[i][0] = -a[0][1] / (a[0][0] - d[i]);
	}
	v[i] /= sqrt(v[i].sum2());	// normalize
	if (v[i][i] < 0.0) v[i] *= (-1.0);
      }
    }
  }
  return;
}
