/*************************************************************01.04.2008*/
/* lin_gauss: solution of a system of linear equations by elimination	*/
/*									*/
/* In each elimination cycle the absolutely largest column element is	*/
/* chosen as pivot. The input arrays are not changed.			*/
/*======================================================================*/
/* a		(i)  coefficient matrix
   b		(i)  right hand side of the system of equations
   RESULT	(o)  result vector
   exceptions	     singular:   singular coefficient matrix
   		     square:     system matrix not square
		     dimensions: vector/matrix sizes incompatible	*/
/************************************************************************/
#include <mathC.h>

using namespace std;


Vector<double> lin_gauss(const Matrix<double>& a, const Vector<double>& b) {
  int i, k, imax, elim, ih,
    n = a.rows();
  double pivot, f;
  Vector<int> ipv(n);
  Vector<double> x(n);
  Matrix<double> w(n, n + 1);

  if (n != a.cols()) throw MthException(MthException::square);
  if (n != b.size()) throw MthException(MthException::dimensions);

  ipv.ramp(0, 1);			// preset pivot index table
  for (i = 0; i < n; i++) {
    for (k = 0; k < n; k++) w[i][k] = a[i][k];	// copy a to workspace
    w[i][n] = b[i];			// copy b to workspace
  }

  for (elim = 0; elim < n; elim++) {	// begin Gauss elimination
    imax = elim;			// find largest element in column
    pivot = w[ipv[imax]][elim];
    for (i = elim + 1; i < n; i++) {
      if (fabs(w[ipv[i]][elim]) > fabs(pivot)) {
	imax = i;
	pivot = w[ipv[i]][elim];
      }
    }
    if (pivot == 0.0) throw MthException(MthException::singular);
    swap(ipv[elim], ipv[imax]);		// update pivot list
    ih = ipv[elim];
    for (i = 0; i < n; i++) {		// eliminations
      if (i != ih) {
	f = w[i][elim] / pivot;
	for (k = elim; k <= n; k++) w[i][k] -= w[ih][k] * f;
      }
    }
  }

  for (i = 0; i < n; i++) {		// move results to x
    k = ipv[i];
    x[i] = w[k][n] / w[k][i];
  }
  return x;
}
