/*************************************************************01.04.2008*/
/* lin_matinv: in-situ matrix inversion by means of the Gauss-Jordan	*/
/*	       algorithm						*/
/*======================================================================*/
/* a		(io) matrix to be inverted/result matrix
   exceptions	     singular: matrix is singular
		     square:   not a square matrix			*/
/************************************************************************/
#include <mathC.h>

using namespace std;


void lin_matinv(Matrix<double>& a) {
  int i, k, elim, imax, kmax, ih, kh,
    n = a.rows();
  double pivot, f;
  Vector<int> ipv(n), kpv(n);		// pivot element coordinates
  Vector<double> b(n);			// sorting space

  if (n != a.cols()) throw MthException(MthException::square);

  ipv.ramp(0, 1);			// preset pivot lists
  kpv.ramp(0, 1);

  for (elim = 0; elim < n; elim++) {
    imax = kmax = elim;			// find pivot element
    pivot = a[ipv[imax]][kpv[imax]];
    for (i = elim; i < n; i++) {
      for (k = elim; k < n; k++) {
	if (fabs(a[ipv[i]][kpv[k]]) > fabs(pivot)) {
	  imax = i;
	  kmax = k;
	  pivot = a[ipv[i]][kpv[k]];
        }
      }
    }
    if (pivot == 0.0) throw MthException(MthException::singular);
    swap(ipv[elim], ipv[imax]);		// update pivot index tables
    ih = ipv[elim];
    swap(kpv[elim], kpv[kmax]);
    kh = kpv[elim];
    for (i = 0; i < n; i++) {		/* Gauss-Jordan transformation */
      if (i != ih) {
	f = a[i][kh] / pivot;
	for (k = 0; k < n; k++) {
	  if (k != kh)
	    a[i][k] -= a[ih][k] * f;
	  else
	    a[i][k] = f;
	}
      }
    }
    for (k = 0; k < n; k++) {
      if (k != kh) a[ih][k] /= -pivot;
    }
    a[ih][kh] = 1.0 / pivot;
  }

  for (i = 0; i < n; i++) {		// sort matrix
    for (k = 0; k < n; k++) b[ipv[k]] = a[i][kpv[k]];
    for (k = 0; k < n; k++) a[i][k] = b[k];
  }
  for (k = 0; k < n; k++) {
    for (i = 0; i < n; i++) b[kpv[i]] = a[ipv[i]][k];
    for (i = 0; i < n; i++) a[i][k] = b[i];
  }
}
