/*************************************************************26.12.2001*/
/* min_brent: minimum of a 1-dimensional function by Brent's method	*/
/*									*/
/* NOTE: Convergence is guaranteed if the ordinate of the middle init-	*/
/*	 ial value is lower than the ordinate values of the others.	*/
/*	 If more than one local minimum exists in the range defined by	*/
/*	 the initial values, the minimum found may not be the deepest	*/
/*	 one.								*/
/*======================================================================*/
/* y = f(x, s)	(i)  function to be minimized
   s		(i)  structure passing additional data to f()
   xa, xb, xc	(i)  initial abscissa values (can be in any order)
   eps		(i)  desired absolute x accuracy
   xmin		(o)  x value at minimum
   RESULT	(o)  y value at minimum
   exceptions	     minimum: no minimum between initial data points
   		     exceptions thrown by f()				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


const double
  gold = 1.618034,			// sectio aurea ratio
  cgold = 2.0 - gold;


double min_brent(function<double(double, void *)> f, void *s, double xa,
double xb, double xc, double eps, double& xmin) {
  double e, e_old, d;
  point2 a, b, c, p;

  if (xa > xb) swap(xa, xb);		// sort initial abscissa values
  if (xb > xc) swap(xb, xc);
  if (xa > xb) swap(xa, xb);
  
  a.x = xa;				// calc. and check ordinates
  a.y = f(a.x, s);
  b.x = xb;
  b.y = f(b.x, s);
  c.x = xc;
  c.y = f(c.x, s);
  if (b.y > a.y || b.y > c.y) throw MthException(MthException::minimum);

  e = d = mth_high;
  while (c.x - a.x > eps) {
    try {
      p.x = min_parabola(a, b, c);	// trial parabolic extrapol.
    }
    catch (MthException& e) {
      if (e.number() != MthException::duplicate) throw;
      break;				 // interval already too narrow
    }
    e_old = e;
    e = d;
    d = p.x - b.x;
    if (p.x <= a.x || p.x >= c.x || fabs(d) > 0.5 * fabs(e_old)) {
      e = ((b.x >= 0.5 * (a.x + c.x)) ? a.x : c.x) - b.x;
      d = cgold * e;			// sectio aurea
      p.x = b.x + d;
    }
    p.y = f(p.x, s);
    if (p.y <= b.y) {			// p lower than previous best?
      if (p.x >= b.x)
	a = b;
      else
	c = b;
      b = p;
    }
    else {
      if (p.x >= b.x)
	c = p;
      else
	a = p;
    }
  }
  xmin = b.x;
  return b.y;
}

/*************************************************************21.02.2006*/
/* min_parabola: abscissa of the minimum of a parabola through 3 points	*/
/*									*/
/* NOTE: We do not check whether the result is a minimum or a maximum.	*/
/*	 Better make sure that a minimum exists before invoking this	*/
/*	 subroutine!							*/
/*======================================================================*/
/* a, b, c	(i)  points
   RESULT	(o)  abscissa value at parabola minimum
   exceptions	     collinear: points collinear
   		     duplicate: identical x values			*/
/************************************************************************/
double min_parabola(point2 a, point2 b, point2 c) {
  double dax, dcx, r, q;

  dax = b.x - a.x;
  r = dax * (b.y - c.y);
  dcx = b.x - c.x;
  q = dcx * (b.y - a.y);
  if (fabs(r) == 0.0 && fabs(q) == 0.0)
    throw MthException(MthException::duplicate);
  if (fabs(q - r) < mth_eps1 * fabs(q + r))
    throw MthException(MthException::collinear);
  return b.x - (dcx * q - dax * r) / (2.0 * (q - r));
}
