/*************************************************************24.01.2003*/
/* ran_cgr48: random number generator (congruential method) with reduced*/
/*	      sequential correlation					*/
/*									*/
/* This routine uses a congruential algorithm developed by Hewlett-	*/
/* Packard (drand48(3C)), which randomizes 48 bit instead of the usual	*/
/* 32, and destroys sequential correlation by means of the shuffling	*/
/* method of Bays and Durham.						*/
/* Ref.: W.H. Press, B.P. Flannery, S.A. Teukolsky, W.T. Vetterling,	*/
/*	 "Numerical Recipes", Cambridge University Press, Cambridge	*/
/*	 1986, section 7.1						*/
/*									*/
/* congruential random number generator: x[i+1] = (a*x[i]+c) mod m	*/
/* HP implementation: a = 0x5deece66d, c = 11, m = 2^48			*/
/* "Recipes" values:  a = 150889, c = 1366, m = 714025			*/
/*									*/
/* compiling: needs option -Ae under HP-UX				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


const int ns = 97;			// length of shuffling table
unsigned long long randint;


static double drand();


double ran_cgr48() noexcept {
  int i;
  static int init = 0;
  static double r[ns], y;

  if (init == 0) {			// initialize subroutine
    init = 1;
    randint = ran_seed();
    for (i = 0; i < ns; i++) r[i] = drand();
    y = drand();
  }
  /* calculation of a random number without sequential correlation:
     use previous random number to select a table entry */
  i = static_cast<int>((ns - 1) * y);
  y = r[i];
  r[i] = drand();
  return y;
}

/************************************************************************/
/* drand: auxiliary random number generator (congruential method)	*/
/************************************************************************/
static double drand() {
  const unsigned long long
    cgr_a = 0x5deece66dLL,		// for congruence algorithm
    cgr_c = 0xbLL,
    cgr_m = 0x1000000000000LL;		// 2^48
  const double cgr_f = (1.0/((double) cgr_m));

  randint = (randint * cgr_a + cgr_c) >> 16;
  return cgr_f * randint;
}
