/*************************************************************14.08.1998*/
/* ran_shuffle: random number generator with reduced sequential	corre-	*/
/*		lation							*/
/*									*/
/* This routine uses a congruential algorithm from the standard		*/
/* C library and destroys sequential correlation by means of the	*/
/* shuffling method of Bays and Durham.					*/
/* [1] W.H. Press, B.P. Flannery, S.A. Teukolsky, W.T. Vetterling,	*/
/*     "Numerical Recipes", Cambridge University Press, Cambridge	*/
/*     1986, section 7.1						*/
/*									*/
/* congruential random number generator: x[i+1] = (a*x[i]+c) mod m	*/
/* this implementation: a = 0x5deece66d, c = 11, m = 2^48		*/
/* "Recipes" values:    a = 150889, c = 1366, m = 714025		*/
/*======================================================================*/
/* RESULT		(o)  random number equidistributed over [0, 1]	*/
/************************************************************************/
#include <mathC.h>

using namespace std;


const int ntab = 97;				// size of shuffling table


double ran_shuffle() noexcept {
  int i;
  static int linit = 0;
  static double r[ntab], y;

  if (linit == 0) {			// initialize subroutine
    linit = 1;
    i = ran_seed();
    srand48(i);				// seed internal function
    for (i = 0; i < ntab; i++) r[i] = drand48();
    y = drand48();
  }
  /* calculation of a random number without sequential correlation:
     use previous random number to select a table entry
  */
  i = static_cast<int>((ntab - 1) * y);
  y = r[i];
  r[i] = drand48();
  return y;
}
