/*************************************************************07.11.2010*/
/* sort_index: determine the order of the elements of a vector		*/
/*	       (ascending order), using the "merge sort" method		*/
/*======================================================================*/
/* a		(i)  vector whose elements need to be sorted
   s		(o)  integer sequence vector
   		     a[s[0]] is the smallest element of a, a[s[1]] the
		     second smallest, etc.				*/
/************************************************************************/
#include <mathC.h>

using namespace std;


static void merge(const Vector<double>& a, int ladr, int lsize, int radr,
int rsize, Vector<int>& s);
static void sort_part(const Vector<double>& a, Vector<int>& s, int p,
int np);


void sort_index(const Vector<double>& a, Vector<int>& s) noexcept {
  int n = a.size();

  if (n == 0) return;

  s.size(n);
  s.ramp(0, 1);
  sort_part(a, s, 0, n);
}

/*************************************************************07.11.2010*/
/* merge: merge two subarrays						*/
/*======================================================================*/
/* a		(i)  vector to be sorted
   ladr, radr	(i)  start addresses of part vectors
   lsize, rsize (i)  their lengths
   s		(o)  combined index vector				*/
/************************************************************************/
static void merge(const Vector<double>& a, int ladr, int lsize, int radr,
int rsize, Vector<int>& s) {
  int i,
    lindex = 0,
    rindex = 0;
  Vector<int> left(lsize), right(rsize);
  
  for (i = 0; i < lsize; i++) left[i] = s[ladr + i];
  for (i = 0; i < rsize; i++) right[i] = s[radr + i];

  i = ladr;
  while (lindex < lsize || rindex < rsize) {
    if (lindex < lsize && rindex < rsize) {
      if (a[left[lindex]] <= a[right[rindex]])
	s[i++] = left[lindex++];
      else
	s[i++] = right[rindex++];
    }
    else if (lindex < lsize)
      s[i++] = left[lindex++];
    else if (rindex < rsize)
      s[i++] = right[rindex++];
  }
}

/*************************************************************07.11.2010*/
/* sort_part: take a part of an index vector and sort it		*/
/*======================================================================*/
/* a		(i)  vector to be sorted
   s		(io) index vector
   p, np	(i)  start index and size of part vector		*/
/************************************************************************/
static void sort_part(const Vector<double>& a, Vector<int>& s, int p,
int np) {
  int half;

  if (np <= 1) return;

  half = np / 2;
  sort_part(a, s, p, half);
  sort_part(a, s, p + half, np - half);
  merge(a, p, half, p + half, np - half, s);
}
