/************************************************************************/
/* test_mcx2: test difficult functions of class multicomplex		*/
/************************************************************************/
#define MAIN
#include <mathC.h>


int main() {
  int i, j, n;
  double a0, b0, x, xe, dl;
  multicomplex a1, b1, a2, b2, a3, b3;
  const double h = 1.0e-20;

  a1.init(1);
  b1.init(1);
  a2.init(2);
  b2.init(2);
  a3.init(3);
  b3.init(3);
  fprintf(stderr, "argument:\n");
  scanf("%lf", &x);
  j = log(x);
  dl = static_cast<double>(j);
  xe = exp(-dl);

  // ln(x) by Taylor expansion
  fprintf(stderr, "max. order of Taylor expansion:\n");
  scanf("%d", &n);
  if (n % 2 == 0) n++;			// should be odd

  a0 = (x * xe - 1.0) / (x * xe + 1.0);		// 0th order
  for (i = n - 2, b0 = 1.0 / n; i >= 1; i -= 2) {
    b0 *= sq(a0);
    b0 += 1.0 / i;
  }
  b0 *= 2.0 * a0;
  b0 += dl;
  printf("f(x):    %12.5e\n", b0);

  a1.r_idx(0, x);			// 1st order
  a1.r_idx(1, h);
  a1 = (a1 * xe - 1.0) / (a1 * xe + 1.0);
  for (i = n - 2, b1 = 1.0 / n; i >= 1; i -= 2) {
    b1 *= sq(a1);
    b1 += 1.0 / i;
  }
  b1 *= 2.0 * a1;
  b1 += dl;
  printf("f'(x):   %12.5e\n", b1.r_idx(1) / h);

  a2.r_idx(0, x);			// 2nd order
  a2.r_idx(1, h);
  a2.r_idx(2, h);
  a2 = (a2 * xe - 1.0) / (a2 * xe + 1.0);
  for (i = n - 2, b2 = 1.0 / n; i >= 1; i -= 2) {
    b2 *= sq(a2);
    b2 += 1.0 / i;
  }
  b2 *= 2.0 * a2;
  b2 += dl;
  printf("f\"(x):   %12.5e\n", b2.r_idx(3) / sq(h));

  a3.r_idx(0, x);			// 3rd order
  a3.r_idx(1, h);
  a3.r_idx(2, h);
  a3.r_idx(4, h);
  a3 = (a3 * xe - 1.0) / (a3 * xe + 1.0);
  for (i = n - 2, b3 = 1.0 / n; i >= 1; i -= 2) {
    b3 *= sq(a3);
    b3 += 1.0 / i;
  }
  b3 *= 2.0 * a3;
  b3 += dl;
  printf("f\"'(x):  %12.5e\n", b3.r_idx(7) / cb(h));

  // sqrt() by Newton iteration
  double y, yold;
  y = mth_high;
  b1 = a1;
  do {
    yold = y;
    b1 = 0.5 * (b1 + a1 / b1);
    y = b1.r_idx(1) / h;
  } while (fabs(y - yold) > fabs(y) * 1.0e-12);
  printf("2r(x):  (%12.5e %12.5e)\n", b1.r_idx(0), b1.r_idx(1));
  printf("2r'(x):  %12.5e\n", b1.r_idx(1) / h);
  b1 *= b1;
  printf("x:      (%12.5e %12.5e)\n", b1.r_idx(0), b1.r_idx(1));

  y = mth_high;
  b2 = a2;
  do {
    yold = y;
    b2 = 0.5 * (b2 + a2 / b2);
    y = b2.r_idx(3) / sq(h);
  } while (fabs(y - yold) > fabs(y) * 1.0e-12);
  printf("2r(x):  (%12.5e %12.5e %12.5e %12.5e)\n", b2.r_idx(0),
  b2.r_idx(1), b2.r_idx(2), b2.r_idx(3));
  printf("2r\"(x):  %12.5e\n", b2.r_idx(3) / sq(h));
  b2 *= b2;
  printf("x:      (%12.5e %12.5e %12.5e %12.5e)\n", b2.r_idx(0),
  b2.r_idx(1), b2.r_idx(2), b2.r_idx(3));

  y = mth_high;
  b3 = a3;
  do {
    yold = y;
    b3 = 0.5 * (b3 + a3 / b3);
    y = b3.r_idx(7) / cb(h);
  } while (fabs(y - yold) > fabs(y) * 1.0e-12);
  printf("2r(x):  (%12.5e %12.5e %12.5e %12.5e\n", b3.r_idx(0),
  b3.r_idx(1), b3.r_idx(2), b3.r_idx(3));
  printf("         %12.5e %12.5e %12.5e %12.5e)\n", b3.r_idx(4),
  b3.r_idx(5), b3.r_idx(6), b3.r_idx(7));
  printf("2r\"'(x): %12.5e\n", b3.r_idx(7) / cb(h));
  b3 *= b3;
  printf("x:      (%12.5e %12.5e %12.5e %12.5e\n", b3.r_idx(0),
  b3.r_idx(1), b3.r_idx(2), b3.r_idx(3));
  printf("         %12.5e %12.5e %12.5e %12.5e)\n", b3.r_idx(4),
  b3.r_idx(5), b3.r_idx(6), b3.r_idx(7));

  // log() via log1p()
  double x00, x01, x10, x11;
  fprintf(stderr, "bicomplex argument:\n");
  scanf("%lf%lf%lf%lf", &x00, &x01, &x10, &x11);
  a2.r_idx(0, x00);
  a2.r_idx(1, x01);
  a2.r_idx(2, x10);
  a2.r_idx(3, x11);
  b2 = log1p(a2);
  printf("ln1p:   (%12.5e %12.5e %12.5e %12.5e)\n", b2.r_idx(0),
  b2.r_idx(1), b2.r_idx(2), b2.r_idx(3));
  b2 = exp(b2);
  printf("e^ln1p: (%12.5e %12.5e %12.5e %12.5e)\n", b2.r_idx(0),
  b2.r_idx(1), b2.r_idx(2), b2.r_idx(3));

  // atan()
  b2 = atan(a2);
  printf("arctan: (%12.5e %12.5e %12.5e %12.5e)\n", b2.r_idx(0),
  b2.r_idx(1), b2.r_idx(2), b2.r_idx(3));
  b2 = sin(b2) / cos(b2);
  printf("tan(at):(%12.5e %12.5e %12.5e %12.5e)\n", b2.r_idx(0),
  b2.r_idx(1), b2.r_idx(2), b2.r_idx(3));
  a1.r_idx(0, x00);
  a1.r_idx(1, x01);
  b1 = atan(a1);
  printf("arctan: (%12.5e %12.5e)\n", b1.r_idx(0), b1.r_idx(1));
  b1 = sin(b1) / cos(b1);
  printf("tan(at):(%12.5e %12.5e)\n", b1.r_idx(0), b1.r_idx(1));
  return mth_GOOD;
}
