/************************************************************************/
/* test_lm1: test differentiation subroutine diff_lm1			*/
/* (calculation of derivatives of exp(-x^2))				*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin:
    (1) argument x, integration radius r, max. order of derivative n	*/
/************************************************************************/
#define MAIN
#include <mathC.h>

#define	USE_LAMBDA	true

#if !USE_LAMBDA
static double f(const complex_d& x, void *u);
#endif


int main() {
  int i, n;
  double x, y, r;

  scanf("%lf%lf%d", &x, &r, &n);
#if !USE_LAMBDA
  printf("f(x)    = %15.8e\n", f(complex_d(x, 0.0), NULL));
#endif
  for (i = 1; i <= n; i++) {
    try {
#if USE_LAMBDA
      y = diff_lm([](const complex_d& z, void *s) -> double {
	return real(exp(-sq(z)));
      }, NULL, i, x, r, -1.0e-10);
#else
      y = diff_lm(f, NULL, i, x, r, -1.0e-10);
#endif
      printf("f[%d](x) = %15.8e\n", i, y);
    }
    catch (MthException& e) {
      printf("error: %s\n", e.message());
      break;
    }
  }
  return mth_GOOD;
}

#if !USE_LAMBDA
/************************************************************************/
/* argument function: exp(-z^2)						*/
/*======================================================================*/
/* z		(i)  argument
   RESULT	(o)  real part of exp(-z^2)				*/
/************************************************************************/
static double f(const complex_d& z, void *u) {
  return std::real(exp(-sq(z)));
}
#endif
