/************************************************************************/
/* test_mcx4: concrete tests of multicomplex derivatives		*/
/*	      contributed by Ian H. Bell				*/
/************************************************************************/
#define MAIN
#include <mathC.h>

static multicomplex f(const Vector<multicomplex>& z, void *u) {
  return sin(z[0] * z[1]);
}

int main() {
  multicomplex a, b;
  double h = 1.0e-100,
    x = 0.1234,
    y = 9876000;
  
  a.init(2);
  b.init(2);
  a.r_idx(0, x);
  a.r_idx(1, h);
  b.r_idx(0, y);
  b.r_idx(2, h);

  // 1st derivative of sin(x) w.r.t. x
  double deriv_num = sin(a).r_idx(1) / h,
    deriv_ana = cos(x);
  printf("a: (%15.8e %15.8e error: %15.8e)\n", deriv_num, deriv_ana,
  deriv_ana - deriv_num);

  // cross derivative of sin(x*y) w.r.t. x and y
  Vector<unsigned int> ord(2);
  ord[0] = 1;
  ord[1] = 1;
  Vector<double> xvec(2);
  xvec[0] = x;
  xvec[1] = y;
  deriv_num = diff_mcx(f, NULL, ord, xvec);
  deriv_ana = -x * y * sin(x * y) + cos(x * y);
  printf("a: (%15.8e %15.8e error: %15.8e)\n", deriv_num, deriv_ana,
  deriv_ana-deriv_num);

  // cross derivative of sin(x*y) w.r.t. x and y
  deriv_num = sin(a * b).r_idx(3) / (h * h);
  deriv_ana = -x * y * sin(x * y) + cos(x * y);
  printf("a: (%15.8e %15.8e error: %15.8e)\n", deriv_num, deriv_ana,
  deriv_ana - deriv_num);

  return mth_GOOD;
}
