/************************************************************************/
/* test_st: test subroutine eqn_steffensen (nonlinear 1D equation	*/
/*	    solver) by applying it to cubic polynomials			*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin sequence:
   1. polynomial coefficents a[0] ... a[3]
   2. initial value							*/
/************************************************************************/
#define MAIN
#include <mathC.h>


static double sf(double x, void *s);


int main() {
  double x, a[4];

  scanf("%lf%lf%lf%lf", &a[0], &a[1], &a[2], &a[3]);
  scanf("%lf", &x);
  try {
    x = eqn_steffensen(sf, a, x, 20, 1.0e-9);
  }
  catch (MthException &e) {
    printf("error: %s\n", e.message());
    return mth_FAIL;
  }
  printf("%f\n", x);
  return mth_GOOD;
}

/************************************************************************/
/* sf: step function defining the equation to solve			*/
/*     here: x = cbrt(-(a[2]*x^2 + a[1]*x + a[0]) / a[3])		*/
/************************************************************************/
static double sf(double x, void *s) {
  double h, y, *a;

  a = static_cast<double *>(s);
  h = -((a[2] * x + a[1]) * x + a[0]) / a[3];
  if (h > 0.0)
    y = pow(h, 1.0 / 3.0);
  else if (h == 0.0)
    y = 0.0;
  else
    y = -pow(-h, 1.0 / 3.0);
  return y;
}
