/************************************************************************/
/* test_fit: test the performance of parameter fitting routines		*/
/*									*/
/* The program can be used for fit_marquardt, fit_powell, and		*/
/* fit_simplex; change the preprocessor statements below as required.	*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin:
     1 initial values for amplitude, center, and width of Gaussian
     2 5 data pairs							*/
/************************************************************************/
#define MAIN
#include <mathC.h>

#define MARQUARDT	1
#define POWELL		0
#define SIMPLEX		0


struct exp_data {
  int n;
  double t[5], u[5];
};


Vector<double> f(const Vector<double>& x, void *uv);


int main() {
  int i;
  double ynorm;
  struct exp_data d;
  Vector<double> x(3), dx(3), y(5);

  x.read(stdin, "%lf");
  for (i = 0; i < 5; i++) scanf("%lf%lf", &d.t[i], &d.u[i]);
  d.n = 5;
  y = f(x, &d);
  printf("before:\n");
  for (i = 0; i < 5; i++) printf("%d %9.6f %9.6f %17.8e\n", i,
    d.t[i], d.u[i], y[i]);
  try {
#if MARQUARDT
  ynorm = fit_marquardt(f, x, 5, &d, 50, 1);
#elif POWELL
  ynorm = fit_powell(f, x, 5, &d, 50, 1);
#else
  dx = 1.0;
  ynorm = fit_simplex(f, x, dx, 5, &d, 50, 1);
#endif
  }
  catch (MthException& e) {
    printf("error: %s\n", e.message());
  }
  y = f(x, &d);
  ynorm = sqrt(y.sum2() / 5);
  printf("\nrms: %e\n", ynorm);
  for (i = 0; i < 3; i++) printf("%f ", x[i]);
  printf("\n");
  for (i = 0; i < 5; i++) printf("%d %9.6f %9.6f %17.8e\n",
    i, d.t[i], d.u[i], y[i]);
  return mth_GOOD;
}

Vector<double> f(const Vector<double>& x, void *uv) {
  int i;
  struct exp_data& a = *static_cast<struct exp_data *>(uv);
  Vector<double> y(5);

  for (i = 0; i < 5; i++) y[i] = x[0] * exp(-sq((a.t[i] - x[1]) / x[2]))
    - a.u[i];
  return y;
}
