/************************************************************************/
/* test_fit_p: test the performance of parameter fitting routines	*/
/*	       fit data to a polynomial					*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin:
    (1)	degree of polynomial
    (2)	number of data points
    (3)	as many data pairs						*/
/************************************************************************/
#define MAIN
#include <mathC.h>


struct exp_data {
  Vector<double> *t, *u;
};


Vector<double> f(const Vector<double>& x, void *uv);


int main() {
  int i, m, n;
  double ynorm;
  struct exp_data d;

  fprintf(stderr, "degree:\n");
  scanf("%d", &m);
  fprintf(stderr, "number of points:\n");
  scanf("%d", &n);
  Vector<double> x(m + 1), t(n), u(n), y(n);
  d.t = &t;
  d.u = &u;

  fprintf(stderr, "%d data pairs:\n", n);
  for (i = 0; i < n; i++) scanf("%lf%lf", &t[i], &u[i]);
  x = 1.0;

  try {
    ynorm = fit_marquardt(f, x, n, &d, 20 * (m + 1), 0);
  }
  catch (MthException& e) {
    printf("error: %s\n", e.message());
  }
  y = f(x, &d);
  ynorm = sqrt(y.sum2() / n);
  printf("\nrms: %e\n", ynorm);
  x.print(stdout, "coefficients:", " %13.6e", "\n");
  printf("\n");
  for (i = 0; i < n; i++) printf("%d %13.6e %13.6e   %13.6e\n", i, t[i],
  u[i], y[i]);
  return mth_GOOD;
}

Vector<double> f(const Vector<double>& x, void *uv) {
  struct exp_data& a = *static_cast<struct exp_data *>(uv);
  int i,
    n = a.t->size();
  Vector<double> y(n);

  for (i = 0; i < n; i++) y[i] = x.polynomial((*a.t)[i]) - (*a.u)[i];
  return y;
}
