/************************************************************************/
/* test_ck2: test solver for a set of ordinary differential equations,	*/
/*	     ode_ck(), vector version					*/
/* (solve {a' = -a, b' = +a} with {a(0) = 1, b(0) = 0}			*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin:
    (1)	integration limit tmax, tabulation step dt.			*/
/************************************************************************/
#define MAIN
#include <mathC.h>


Vector<double> f(double x, const Vector<double>& y, void *s);


int main() {
  int i;
  double tmax, dt, t0;
  Vector<double> y0(2), y(2);

  scanf("%lf%lf", &tmax, &dt);
  if (tmax * dt < 0.0) {
    printf("tmax and dt must have equal signs.\n");
    return mth_FAIL;
  }

  t0 = 0.0;
  y0[0] = 1.0;
  y0[1] = 0.0;
  printf("          t           a           b     exp(-x)\n");
  printf("%11.4e %11.4e %11.4e %11.4e\n", t0, y0[0], y0[1], 1.0);

  while ((t0 - tmax) * dt < 0.0) {
    y = ode_ck(f, NULL, t0, y0, t0 + dt, 0.0);
    t0 += dt;
    y0 = y;
    printf("%11.4e %11.4e %11.4e %11.4e\n", t0, y[0], y[1], exp(-t0));
  }
  return mth_GOOD;
}

/************************************************************************/
/* ODE function: y' = f(t, y)						*/
/*======================================================================*/
/* t, y		(i)  arguments
   s		(i)  pass-through argument
   RESULT	(o)  -y							*/
/************************************************************************/
Vector<double> f(double t, const Vector<double>& y, void *s) {
  Vector<double> dy(2);
  dy[0] = -y[0];
  dy[1] = y[0];
  return dy;
}
