/************************************************************************/
/* test_rk: test solver for ordinary differential equations		*/
/*	    ode_runge_kutta()						*/
/* (solve y' + y = 0, with y(0) = 1)					*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin:
    (1)	integration limit xmax, tabulation step dx.			*/
/************************************************************************/
#define MAIN
#include <mathC.h>


double f(double x, double y, void *s);


int main() {
  double xmax, dx, x0, y0, y;

  scanf("%lf%lf", &xmax, &dx);
  if (xmax * dx < 0.0) {
    printf("xmax and dx must have equal signs.\n");
    return mth_FAIL;
  }

  x0 = 0.0;
  y0 = 1.0;
  printf("          x           y     exp(-x)\n");
  printf("%11.4e %11.4e %11.4e\n", x0, y0, 1.0);

  while ((x0 - xmax) * dx < 0.0) {
    y = ode_runge_kutta(f, NULL, x0, y0, x0 + dx, 0.0);
    x0 += dx;
    y0 = y;
    printf("%11.4e %11.4e %11.4e\n", x0, y0, exp(-x0));
  }
  return mth_GOOD;
}

/************************************************************************/
/* ODE function: y' = f(x, y)						*/
/*======================================================================*/
/* x, y		(i)  arguments
   s		(i)  pass-through argument
   RESULT	(o)  -y							*/
/************************************************************************/
double f(double x, double y, void *s) {
  return (-y);
}
