/*************************************************************04.09.2001*/
/* array_ops: simple vector operations (C library version)		*/
/*======================================================================*/
/* n		(i)  vector size					*/
/* v1[n], v2[n]	(i)  input vectors					*/
/* v3[n]	(o)  output vector					*/
/* a		(i)  scalar operand					*/
/* f(x)		(i)  1-dimensional function				*/
/*									*/
/* operations:	+                              subroutine: vadd		*/
/*		-                                          vsub		*/
/*		* (element by element)                     vmult	*/
/*		/                                          vdiv		*/
/*		scalar multiplication v3=v1*a              vsmult	*/
/* 		scalar addition                            vsadd	*/
/*		scalar assignment v3=(a)                   vset		*/
/*		copy                                       vcopy	*/
/*		function evaluation v3=f(v1)               vfunc	*/
/************************************************************************/
#include <mathc.h>

void vadd(int n, double *v1, double *v2, double *v3) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++ + *v2++;
}
 
void vsub(int n, double *v1, double *v2, double *v3) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++ - *v2++;
}
 
void vmult(int n, double *v1, double *v2, double *v3) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++ * *v2++;
}
 
void vdiv(int n, double *v1, double *v2, double *v3) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++ / *v2++;
}
 
void vsmult(int n, double *v1, double *v3, double a) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++ * a;
}
 
void vsadd(int n, double *v1, double *v3, double a) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++ + a;
}
 
void vset(int n, double *v3, double a) {
  int i;
  for (i = 0; i < n; i++) *v3++ = a;
}
 
void vcopy(int n, double *v1, double *v3) {
  int i;
  for (i = 0; i < n; i++) *v3++ = *v1++;
}

void vfunc(int n, double *v1, double *v3, double (*f)()) {
  int i;
  for (i = 0; i < n; i++) *v3++ = (*f)(*v1++);
}

double dot(int n, double *v1, double *v2) {
  int i;
  double a;
  for (i = 0, a = 0.0; i < n; i++) a += *v1++ * *v2++;
  return (a);
}
