/*************************************************************14.06.2002*/
/* diff_x: numerical derivation of a function w.r.t. to a variable	*/
/*	   confined to the interval [0; 1]				*/
/*									*/
/* Close to x = 0 and x = 1 an asymmetric parabolic interpolation	*/
/* is used in order to avoid rounding errors.				*/
/*									*/
/* usage: In order to differentiate the function f(V, T, x, whatever)	*/
/*	  write a wrapper function wrap_f(double x1, struct wrp *s)	*/
/*	  with struct wrp { double V, T; void *whatever};		*/
/*======================================================================*/
/* f		(i)  function to be differentiated, f(x, s)
   aux		(i)  structure for pass-through parameters
   x		(i)  argument
   dx		(i)  differentiation increment
   RESULT	(o)  derivative of f					*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

double diff_x(double (*f)(), void *aux, double x, double dx) {
  double a0, a1, a2, h1, h2;

  dx = fabs(dx);
  if (x >= 1.0 - dx) {
    a0 = (*f)(1.0, aux);
    h1 = (*f)(1.0 - dx, aux);
    h2 = (*f)(1.0 - 2 * dx, aux);
    a1 = -(4 * h1 - h2 - 3 * a0) / (2 * dx);
    a2 = (2 * h1 - h2 -a0) / (2 * SQ(dx));
    return (a1 + 2 * (x - 1.0) * a2);
  }
  else if (x <= dx) {
    a0 = (*f)(0.0, aux);
    h1 = (*f)(dx, aux);
    h2 = (*f)(2 * dx, aux);
    a1 = (4 * h1 - h2 - 3 * a0) / (2 * dx);
    a2 = (2 * h1 - h2 - a0) / (2 * SQ(dx));
    return (a1 + 2 * x * a2);
  }
  else {
    h1 = (*f)(x + dx, aux);
    h2 = (*f)(x - dx, aux);
    return ((h1 - h2) / (2 * dx));
  }
}
