/*************************************************************03.09.2001*/
/* eqn_quadratic: solution of a quadratic equation				*/
/*									*/
/* Equations of lesser degree are solved by the appropriate formulas.	*/
/* The solutions are arranged in ascending order.			*/
/*======================================================================*/
/* a		(i)  array containing the polynomial coefficients
   x		(o)  result array
   RESULT	(o)  number of valid solutions				*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

int eqn_quadratic(const double *a, double *x) {
  int nreal;
  double p, dis;

  if (a[2] != 0.0) {			/* quadratic problem? */
    p = 0.5 * a[1] / a[2];
    dis = SQ(p) - a[0] / a[2];
    if (dis >= 0.0) {			/* two real solutions */
      dis = sqrt(dis);
      x[0] = -p - dis;
      x[1] = -p + dis;
      nreal = 2;
    }
    else				/* no real solution */
      nreal = 0;
  }

  else if (a[1] != 0.0) {		/* linear problem? */ 
    x[0] = -a[0] / a[1];
    nreal = 1;
  }

  else					/* no equation */
    nreal = 0;

  return (nreal);
}
