/*************************************************************09.10.2005*/
/* eqn_regulafalsi: fast 1-dimensional regula falsi iteration (Illinois	*/
/*		    algorithm, a.k.a. pegasus algorithm)		*/
/*									*/
/* The input parameters are not changed. Convergence is certain, if	*/
/* ya and yb have opposite signs.					*/
/*======================================================================*/
/* f(x, y, s)	(i)  object function (y: function value, return: error
		     code)
   s		(i)  structure passing additional data to f()
   xa, xb	(i)  initial values
   ya, yb	(i)  residuals at xa and xb (ya = f(xa), yb = f(xb))
   eps		(i)  desired absolute precision
   x		(o)  x value for which y = 0
   RESULT	(o)  error code:
			MTH_ILL = ya and yb have the same signs
			MTH_OBJ = object function error			*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

int eqn_regulafalsi(int (*f)(), void *s, double xa, double xb, double ya,
double yb, double eps, double *x) {
  int err;
  double dx, h;
  point2 p1, p2, pnew;

  if (ya * yb > 0.0) return MTH_ILL;
  p1.x = xa;
  p1.y = ya;
  p2.x = xb;
  p2.y = yb;
  while (1) {			/* loop terminated from within! */
    dx = p1.y * (p1.x - p2.x) / (p1.y - p2.y);
    pnew.x = p1.x - dx;
    if (fabs(dx) < eps) break;
    if ((err = (*f)(pnew.x, s, &pnew.y)) != 0) return (MTH_OBJ(err));
    h = pnew.y * p1.y;
    if (h < 0.0)
      p2 = p1;
    else if (h == 0.0)
      break;
    else
      p2.y *= 0.5;
    p1 = pnew;
  }
  *x = pnew.x;
  return (MTH_GOOD);
}
