/*************************************************************12.12.2001*/
/* eqn_steffensen: 1-dimensional root finder using Steffensen's		*/
/*		   accelerated substitution algorithm			*/
/*======================================================================*/
/* f(x, u)	(i)  iteration step function (solution x = f(x, u))
   u		(i)  structure for passing additional information to f()
   x		(io) initial value/result
   kmax		(i)  number of iteration steps
   eps		(i)  convergence criterion (max. x deviation)
   RESULT	(o)  error code:
		       MTH_FAIL = algorithm failure (inflection point)	*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

#define XRES		1.E-12		/* machine precision */

int eqn_steffensen(double (*f)(), void *u, double *x, int kmax, double eps) {
  int i;
  double x0, x1, x2, d, ds;

  for (i = 0; i < kmax; i++) {
    x0 = *x;
    x1 = (*f)(x0, u);
    if (fabs(x1 - x0) < XRES * fabs(x0)) return (MTH_GOOD);
    x2 = (*f)(x1, u);
    d = x2 - 2 * x1 + x0;
    if (d == 0.0) return (MTH_FAIL);
    ds = SQ(x1 - x0) / d;
    *x = x0 - ds;
    if (fabs(ds) < eps) return (MTH_GOOD);
  }
  return (MTH_NOCONV);
}
