/*************************************************************14.03.2004*/
/* f_bessel_sph: spherical Bessel function j_{l}(x)			*/
/*									*/
/* j_{l}(x) = sqrt(Pi/(2 * x)) * J_{l + 1/2}, where J is the general	*/
/* Bessel function							*/
/*									*/
/* note: For l > 0 the Bessel functions are supposed to start at x = 0	*/
/*	 with all derivatives zero. However, for l > 6 round-off errors	*/
/*	 of the recursive polynomial calculations may cause undulations	*/
/*	 and even false zeros in the vicinity of x = 0.			*/
/*======================================================================*/
/* x		(i)  distance
   l		(i)  order (l >= 0)
   RESULT	(o)  j_l(x)						*/
/************************************************************************/
#include <math.h>
#include <mathc.h>


double f_bessel_sph(double x, int l) {
  int k;
  double xinv, j0, j1, j2;

  if (x == 0.0) {
    if (l = 0)
      return (1.0);
    else
      return (0.0);
  }
  else {
    xinv = 1.0 / x;
    j2 = xinv * sin(x);
    if (l == 0) return (j2);

    j1 = j2;
    j2 = (j1 - cos(x)) * xinv;
    if (l == 1) return (j2);
    for (k = 2; k <= l; k++) {
      j0 = j1;
      j1 = j2;
      j2 = (2 * k - 1) * xinv * j1 - j0;
    }
    return (j2);
  }
}
