/************************************************************************/
/* Gamma function, factorial, etc.					*/
/************************************************************************/

#include <math.h>
#include <mathc.h>


/*************************************************************25.08.2005*/
/* f_gammaln: logarithm of Gamma function				*/
/*									*/
/* after: Numerical Recipes						*/
/*======================================================================*/
/* x		(i)  argument (must be positive)
   RESULT	(o)  ln(Gamma(x))					*/
/************************************************************************/
double f_gammaln(double x) {
  double y, tmp, ser;
  static double coeff[6] = {76.18009172947146, -86.50532032941677,
    24.01409824083091, -1.231739572450155, 0.1208650973866179E-2,
    -0.5395239384953E-5};
  int j;

  y = x;
  tmp = x + 5.5;
  tmp -= (x + 0.5) * log(tmp);
  ser = 1.000000000190015;
  for (j = 0; j <= 5; j++) ser += coeff[j] / ++y;
  return (-tmp + log(2.5066282746310005 * ser / x));
}

/*************************************************************25.08.2005*/
/* f_dfac: factorial (as floating point)				*/
/*									*/
/* after: Numerical recipes						*/
/*======================================================================*/
/* n		(i)  argument (must >= 0)
   RESULT	(o)  n!							*/
/************************************************************************/
double f_dfac(int n) {
  int i;
  static int nfill = 6;
  static double a[70] = {1.0, 1.0, 2.0, 6.0, 24.0, 120.0, 720.0};

  if (n >= 70) return (exp(f_gammaln(n + 1.0)));

  while (nfill < n) {
    nfill++;
    a[nfill] = a[nfill - 1] * nfill;
  }
  return (a[n]);
}

/*************************************************************25.08.2005*/
/* f_ibinom_c: binomial coefficents (as integers)			*/
/*									*/
/* method: Pascal's triangle, results are remembered			*/
/*======================================================================*/
/* n, i		(i)  arguments
   RESULT	(o)  (n i) ("n over i")
		     A negative value indicates an error.		*/
/************************************************************************/

#define NBCMAX		40		/* largest n value allowed */


f_ibinom_c(int n, int i) {
  int j, *p1, *p2;
  static int nfill = -1, bc[(NBCMAX + 1) * (NBCMAX + 2) / 2];

  if (nfill < 0) {			/* initialize */
    bc[0] = 1;
    nfill = 0;
  }

  if (n < 0 || i < 0 || i > n || n > NBCMAX) return (-1);

  if (n > nfill) {			/* add rows to table */
    p1 = bc + nfill * (nfill + 1) / 2;	/* begin of last filled row */
    p2 = p1 + nfill + 1;		/* begin of 1st empty row */
    while (nfill < n) {
      nfill++;
      *p2++ = 1;
      for (j = 1; j < nfill; j++, p1++, p2++) *p2 = *p1 + *(p1 + 1);
      *p2++ = 1;
      p1++;
    }
  }

  return (bc[n * (n + 1) / 2 + i]);
}
