/*************************************************************07.06.2007*/
/* int_romberg: integration of a function by means of Romberg's method	*/
/*======================================================================*/
/* f(x, u)	(i)  integrand function (x = scalar)
   u		(i)  structure for passing additional information
   a, b		(i)  interval limits
   eps		(i)  desired accuracy: >0 = absolute
				       <0 = relative
		     Warning: If the integral is zero and eps < 0,
		     the subroutine will return error MTH_NOCONV.
   maxord	(i)  maximum order of extrapolation (max. 15)
   q		(o)  integral value
   RESULT	(o)  error code:
		       MTH_NOCONV = no convergence; result not precise	*/
/************************************************************************/
#include <math.h>
#include <mathc.h>


#define MMAX		16


int int_romberg(double (*f)(), void *u, double a, double b, double eps,
int maxord, double *q) {
  int i, j, n;
  double s[2][MMAX], acteps, h, acc, k;

  if (eps >= 0.0) acteps = eps;
  if (maxord > MMAX - 1) maxord = MMAX - 1;

  h = b - a;				/* zeroth approximation: */
  n = 1;				/* a single trapezoid */
  s[1][0] = ((*f)(a, u) + (*f)(b, u)) * 0.5 * h;

  for (j = 1; j <= maxord; j++) {	/* successive approximation schema */
    for (i = 0; i < j; i++) s[0][i] = s[1][i];	/* save old approx. */
    h *= 0.5;				/* new trapezoidal integration */
    n *= 2;				/* with double panel number */
    for (i = 1, acc = 0.0; i < n; i += 2) acc += (*f)(a + i * h, u);
    s[1][0] = 0.5 * s[1][0] + h * acc;
    for (i = 1, k = 1.0; i <= j; i++) {	/* Romberg's extrapolation */
      k *= 4.0;
      s[1][i] = (k * s[1][i-1] - s[0][i-1]) / (k - 1.0);
    }
    *q = s[1][j];
    if (eps < 0.0) acteps = fabs(eps * *q);	/* relative precision */
    if (fabs(s[1][j] - s[0][j-1]) <= acteps) return MTH_GOOD;
  }
  return MTH_NOCONV;			/* desired accuracy not obtained */
}
