/*************************************************************16.12.2001*/
/* ipl_eval: evaluation of the spline function defined by		*/
/*	     subroutines ipl_spline or ipl_akima			*/
/*======================================================================*/
/* xvalue	(i)  abszissa value
   x[n], y[n]	(i)  abszissa and ordinate arrays as used by ipl_spline()
   b, c, d[n]	(i)  coefficient arrays generated by ipl_spline()
   k		(i)  order of derivative (0 <= k <= 2)
   RESULT	(o)  value of spline function				*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

double ipl_eval(double xvalue, int n, double *x, double *y, double *b,
double *c, double *d, int k) {
  int i, high, low;
  double xx, yvalue;

  if (xvalue < x[0]) {
    switch (k) {
    case 0:
      yvalue = y[0] + b[0] * (xvalue - x[0]);
      break;
    case 1:
      yvalue = b[0];
      break;
    default:
      yvalue = 0.0;
    }
  }
  else if (xvalue < x[n - 1]) {		/* locate appropriate interval */
    low = 0;				/* by bisection */
    high = n - 1;
    while (high - low > 1) {
      i = (low + high) / 2;
      if (xvalue >= x[i])
	low = i;
      else
	high = i;
    }
    xx = xvalue - x[low];
    switch (k) {
    case 0:
      yvalue = ((d[low] * xx + c[low]) * xx + b[low]) * xx + y[low];
      break;
    case 1:
      yvalue = (3 * d[low] * xx + 2 * c[low]) * xx + b[low];
      break;
    case 2:
      yvalue = 6 * d[low] * xx + 2 * c[low];
      break;
    case 3:
      yvalue = 6 * d[low];
      break;
    default:
      yvalue = 0.0;
    }
  }
  else {
    switch (k) {
    case 0:
      yvalue = y[n - 1] + b[n - 1] * (xvalue - x[n - 1]);
      break;
    case 1:
      yvalue = b[n - 1];
      break;
    default:
      yvalue = 0.0;
    }
  }
  return (yvalue);
}
