/*************************************************************02.01.2003*/
/* ipl_lagrange: Lagrange interpolation polynomial			*/
/*									*/
/* G. Jordan-Engeln, F. Reutter: Numerische Mathematik fr Ingenieure,	*/
/* BI Hochschultaschenbcher Bd. 104, Bibliographisches Institut,	*/
/* Mannheim 1973, pp. 198ff						*/
/*======================================================================*/
/* n		(i)  number of data points
   x[n], y[n]	(i)  abscissa and ordinate values
   a[n]		(o)  coefficients of Lagrange polynomial
   RESULT	(o)  error code:
		       MTH_FAIL = some x values are identical
		       MTH_NOMEM = could not allocate memory		*/
/************************************************************************/
#include <math.h>
#include <mathc.h>
#include <stdlib.h>

int ipl_lagrange(int n, double *x, double *y, double *a) {
  int i, j, k, err, s;
  double *q, f;

  if ((q = (double *) malloc((size_t) (n * sizeof(double)))) == NULL)
  return (MTH_NOMEM);

  err = 0;
  for (i = 0; i < n; i++) a[i] = 0.0;

  for (k = 0; k < n; k++) {
    q[0] = 1.0;
    for (i = 1; i < n; i++) q[i] = 0.0;
    for (j = s = 0, f = 1.0; j < n; j++) {
      if (j == k) continue;
      s++;
      for (i = s; i >= 1; i--) q[i] = -x[j] * q[i] + q[i - 1];
      q[0] *= -x[j];
      f *= (x[k] - x[j]);
    }
    if (f == 0.0) {
      err = MTH_FAIL;
      break;
    }
    for (i = 0; i < n; i++) a[i] += y[k] * q[i] / f;
  }
  free(q);
  return (err);
}
