/*************************************************************26.12.2001*/
/* min_brent: minimum of a 1-dimensional function by Brent's method	*/
/*									*/
/* NOTE: Convergence is guaranteed if the ordinate of the middle init-	*/
/*	 ial value is lower than the ordinate values of the others.	*/
/*	 If more than one local minimum exists in the range defined by	*/
/*	 the initial values, the minimum found may not be the deepest	*/
/*	 one.								*/
/*======================================================================*/
/* f(x, y, aux)		(i)  function to be minimized
   aux			(i)  structure passing additional data to f()
   xa, xb, xc		(i)  initial abscissa values (can be in any order)
   eps			(i)  desired absolute x accuracy
   xmin, ymin		(o)  minimum coordinates
   RESULT		(o)  error code:
			       MTH_ILL = no minimum
			       MTH_OBJ = error in f()			*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

#define	GOLD		1.618034	/* sectio aurea ratio */
#define CGOLD		(2.0 - GOLD)


int min_brent(int (*f)(), void *aux, double xa, double xb, double xc,
double eps, double *xmin, double *ymin) {
  int err;
  double e, e_old, d;
  point2 a, b, c, p;

  if (xa > xb) SWAP(xa, xb, e);		/* sort initial abscissa values */
  if (xb > xc) SWAP(xb, xc, e);
  if (xa > xb) SWAP(xa, xb, e);
  
  a.x = xa;				/* calc. and check ordinates */
  if ((err = (*f)(a.x, &a.y, aux)) != 0) return (MTH_OBJ(err));
  b.x = xb;
  if ((err = (*f)(b.x, &b.y, aux)) != 0) return (MTH_OBJ(err));
  c.x = xc;
  if ((err = (*f)(c.x, &c.y, aux)) != 0) return (MTH_OBJ(err));
  if (b.y > a.y || b.y > c.y) return (MTH_ILL);

  e = d = DHIGH;
  while (c.x - a.x > eps) {
    err = min_parabola(a, b, c, &p.x);	/* trial parabolic extrapol. */
    if (err = MTH_NOCONV) break;	/* interval already too narrow */
    e_old = e;
    e = d;
    d = p.x - b.x;
    if (err || p.x <= a.x || p.x >= c.x || fabs(d) > 0.5 * fabs(e_old)) {
      e = ((b.x >= 0.5 * (a.x + c.x)) ? a.x : c.x) - b.x;
      d = CGOLD * e;			/* sectio aurea */
      p.x = b.x + d;
    }
    if ((err = (*f)(p.x, &p.y, aux)) != 0) return (MTH_OBJ(err));
    if (p.y <= b.y) {			/* p lower than previous best? */
      if (p.x >= b.x)
	a = b;
      else
	c = b;
      b = p;
    }
    else {
      if (p.x >= b.x)
	c = p;
      else
	a = p;
    }
  }
  *xmin = b.x;
  *ymin = b.y;
  return (MTH_GOOD);
}

/*************************************************************21.02.2006*/
/* min_parabola: abscissa of the minimum of a parabola through 3 points	*/
/*									*/
/* NOTE: We do not check whether the result is a minimum or a maximum.	*/
/*	 Better make sure that a minimum exists before invoking this	*/
/*	 subroutine!							*/
/*======================================================================*/
/* a, b, c	(i)  points
   x		(o)  abscissa value at parabola minimum
   RESULT	(o)  error code:
		       MTH_GOOD: no error
		       MTH_NOCONV: points too closely spaced for
			 evaluation (which means that b.x is probably
			 a sufficiently good approximation)
		       MTH_ILL: points collinear			*/
/************************************************************************/
int min_parabola(point2 a, point2 b, point2 c, double *x) {
  double dax, dcx, r, q;

  dax = b.x - a.x;
  r = dax * (b.y - c.y);
  dcx = b.x - c.x;
  q = dcx * (b.y - a.y);
  if (fabs(r) == 0.0 && fabs(q) == 0.0) {
    *x = b.x;
    return (MTH_NOCONV);
  }
  if (fabs(q - r) < DEPS * fabs(q + r)) {
    *x = (q > r) ? DHIGH : -DHIGH;
    return (MTH_ILL);
  }
  *x = b.x - (dcx * q - dax * r) / (2 * (q - r));
  return (MTH_GOOD);
}
