/*************************************************************16.12.2001*/
/* old_matrix: old version of matrix operation subroutines		*/
/************************************************************************/
#include <stdlib.h>
#include <math.h>
#include <mathc.h>

/*************************************************************14.08.1998*/
/* mpro: matrix product c = a*b						*/
/*======================================================================*/
/* a, b		(i)  input matrices (a[l][m], b[m][n])
   c[l][n]	(o)  output matrix
   l, m, n	(i)  matrix dimensions					*/
/************************************************************************/
void mpro(int l, int m, double *a, int n, double *b, double *c) {
  int i, j, k;
  double sum, *pa, *pb, *bb;
  for (i = 0; i < l; i++, a += m) {
    for (j = 0, bb = b; j < n; j++, bb++) {
      for (k = 0, sum = 0.0, pa = a, pb = bb; k < m; k++) {
	sum += *pa++ * *pb;
	pb += n;
      }
      *c++ = sum;
    }
  }
}

/*************************************************************14.08.1998*/
/* mprot: matrix product c = a * transpose(b)				*/
/*======================================================================*/
/* a, b		(i)  input matrices (a[l][m], b[n][m])
   c[l][n]	(o)  result matrix
   l, m, n	(i)  matrix dimensions					*/
/************************************************************************/
void mprot(int l, int m, double *a, int n, double *b, double *c) {
  int i, j ,k;
  double sum, *pa, *pb, *bb;
  for (i = 0; i < l; i++, a += m) {
    for (j = 0, bb = b; j < n; j++, bb += m) {
      for (k = 0, sum = 0.0, pa = a, pb = bb; k < m; k++) sum += *pa++ * *pb++;
      *c++ = sum;
    }
  }
}

/*************************************************************02.09.2001*/
/* matinv: in-situ matrix inversion by means of Gauss-Jordan algorithm	*/
/*======================================================================*/
/* n		(io) matrix size
   a		(io) matrix to be inverted/result matrix (pointer to
		     n*n block of double numbers)
   RESULT	(o)  error code: 0 = no error
				 1 = matrix is singular
				 2 = memory allocation problem		*/
/************************************************************************/
int matinv(int n, double *a) {
  int i, k, elim, *ipv, *kpv, imax, kmax, *pi, *pk, err;
  double pivot, f, *b, **c, **pc;

  /* memory allocations for pivot element indices and sort space */
  if (create_ivector(&ipv, 0, n-1) != 0 || create_ivector(&kpv, 0, n-1) != 0
  || create_dvector(&b, 0, n-1) != 0) return (2);
  if ((pc = (double **) malloc((size_t) (n * sizeof(double *)))) == NULL)
  return (2);				/* organize a[] as a matrix */
  c = overlay_dmatrix(pc, a, 0, n-1, 0, n-1);

  for (i = 0, pi = ipv, pk = kpv; i < n; i++) *pi++ = *pk++ = i;
  /* pivot list */

  for (elim = 0, err = 0; elim < n; elim++) {/* begin elimination */
    imax = kmax = elim;			/* find pivot element (abs. */
    pivot = c[ipv[imax]][kpv[imax]];	/* largest elmnt. of remaining*/
    for (i = elim; i < n; i++) {	/* matrix) */
      for (k = elim; k < n; k++) {
	if (fabs(c[ipv[i]][kpv[k]]) > fabs(pivot)) {
	  imax = i;
	  kmax = k;
	  pivot = c[ipv[i]][kpv[k]];
        }
      }
    }
    if (pivot == 0.0) {			/* matrix singular? */
      err = 1;
      break;
    }
    pi = ipv + elim;
    i = *pi;				/* record pivot indices */
    *pi = ipv[imax];			/* ... pivot row */
    ipv[imax] = i;
    pk = kpv + elim;
    k = *pk;
    *pk = kpv[kmax];			/* ... pivot column */
    kpv[kmax] = k;
    for (i = 0; i < n; i++) {		/* Gauss-Jordan transformation */
      if (i != *pi) {
	f = c[i][*pk]/pivot;
	for (k = 0; k < n; k++) {
	  if (k != *pk)
	    c[i][k] -= c[*pi][k] * f;
	  else
	    c[i][k] = f;
	}
      }
    }
    for (k = 0; k < n; k++) {
      if (k != *pk) c[*pi][k] /= -pivot;
    }
    c[*pi][*pk] = 1.0/pivot;
  }

  if (err == 0) {
    for (i = 0; i < n; i++) {		/* sort matrix elements */
      for (k = 0, pi = ipv, pk = kpv; k < n; k++) b[*pi++] = c[i][*pk++];
      for (k = 0; k < n; k++) c[i][k] = b[k];
    }
    for (k = 0; k < n; k++) {
      for (i = 0, pi = ipv, pk = kpv; i < n; i++) b[*pk++] = c[*pi++][k];
      for (i = 0; i < n; i++) c[i][k] = b[i];
    }
  }
  free_ivector(ipv, 0);
  free_ivector(kpv, 0);
  free_dvector(b, 0);
  free((void *) pc);
  return (err);
}

/*************************************************************27.05.2001*/
/* linsys: solution of a system of linear equations by Gauss elimination*/
/*									*/
/* In each elimination cycle the absolutely largest column element is	*/
/* chosen as pivot. The input arrays are not changed.			*/
/*======================================================================*/
/* n		(i)  number of equations
   a[n][n]	(i)  coefficient matrix
   b[n]		(i)  right hand side of the system of equations
   x[n]		(o)  result vector
   RESULT	(o)  error code: 0 = no error
				 1 = singular coefficient matrix
				 2 = n is too large			*/
/************************************************************************/
int linsys(int n, double *a, double *b, double *x) {
  int i, k, imax, ielim, *pi, *ipv, err;
  double *pa, *pb, *pw, pivot, f, **w;

  /* allocate memory for sort vector and workspace matrix */
  if (create_ivector(&ipv, 0, n - 1) != 0 
  || create_dmatrix(&w, 0, n - 1, 0, n) != 0) return (2);

  for (i = 0, pw = w[0], pa = a, pb = b, pi = ipv, err = 0; i < n; i++) {
    *pi++ = i;				/* preset pivot table */
    for (k = 0; k < n; k++) *pw++ = *pa++;	/* copy a to workspace */
    *pw++ = *pb++;			/* copy b to workspace */
  }

  for (ielim = 0; ielim < n; ielim++) {	/* begin Gauss elimination */
    imax = ielim;			/* find pivot element */
    pivot = w[ipv[imax]][ielim];
    for (i = ielim; i < n; i++) {
      if (fabs(w[ipv[i]][ielim]) > fabs(pivot)) {
	imax = i;
	pivot = w[ipv[i]][ielim];
      }
    }
    if (pivot == 0.0) {
      err = 1;
      break;
    }
    pi = ipv + ielim;
    i = *pi;				/* update pivot table */
    *pi = ipv[imax];
    ipv[imax] = i;
    for (i = 0; i < n; i++) {		/* eliminations */
      if (i != *pi) {
	f = w[i][ielim]/pivot;
	for (k = ielim; k <= n; k++) w[i][k] -= w[*pi][k] * f;
      }
    }
  }

  if (err == 0) {			/* move results to x vector */
    for (i = 0; i < n; i++) *x++ = w[ipv[i]][n]/w[ipv[i]][i];
  }
  free_ivector(ipv, 0);
  free_dmatrix(w, 0, 0);
  return (err);
}

/*************************************************************30.08.2001*/
/* fixed_matrix: overlay a 1-dimensional array with vectors or matrices	*/
/*		 having user-defined index ranges			*/
/*									*/
/* This is almost the same as the dynamic-matrix package (dmatrix.c),	*/
/* except that here no dynamic memory allocations are used.		*/
/*									*/
/* usage:								*/
/*   double **a, *(apointer[3]), aspace[12];				*/
/*   a = overlay_dmatrix(apointer, aspace, -1, 1, 1, 4);		*/
/*   h = a[-1][1] + a[-1][4] + a[1][1] + a[1][4];			*/
/*									*/
/* WARNING: The array boundary ranges must match the array size. There	*/
/*	    is no way to check this at runtime!				*/
/************************************************************************/

/*************************************************************30.08.2001*/
/* overlay_dmatrix: overlay a double array with a matrix		*/
/*======================================================================*/
/* p		(io)  array of pointers (1 for each row)
   a		(i)   array to be organized as a matrix
   nrl, nrh	(i)   lower and upper row boundaries
   ncl, nch	(i)   lower and upper column boundaries
   RESULT	(o)   pointer to p					*/
/************************************************************************/
double **overlay_dmatrix(double **p, double *a, int nrl, int nrh,
int ncl, int nch) {
  int i, ncol;
  double **m;

  ncol = nch - ncl + 1;
  m = p - nrl;
  *(m + nrl) = a - ncl;
  for (i = nrl + 1; i <= nrh; i++) *(m + i) = *(m + i -1) + ncol;
  return (m);
}

/*************************************************************30.08.2001*/
/* overlay_imatrix: overlay an integer array with a matrix		*/
/*======================================================================*/
/* p		(io)  array of pointers (1 for each row)
   a		(i)   array to be organized as a matrix
   nrl, nrh	(i)   lower and upper row boundaries
   ncl, nch	(i)   lower and upper column boundaries
   RESULT	(o)   pointer to p					*/
/************************************************************************/
int **overlay_imatrix(int **p, int *a, int nrl, int nrh, int ncl, int nch) {
  int i, ncol;
  int **m;

  ncol = nch - ncl + 1;
  m = p - nrl;
  *(m + nrl) = a - ncl;
  for (i = nrl + 1; i <= nrh; i++) *(m + i) = *(m + i -1) + ncol;
  return (m);
}

/*************************************************************31.08.2001*/
/* dynamic_matrix: dynamic matrix and vector allocations with 		*/
/* user-defined subscripts						*/
/* Ref.: W.H. Press, B.P. Flannery, S.A. Teukolsky, W.T. Vetterling,	*/
/* Numerical Recipes in C, Cambridge University Press, 1992		*/
/************************************************************************/

/*************************************************************27.10.1995*/
/* create_dmatrix: allocate a double matrix with user-defined subscript	*/
/*		   range: m[nrl..nrh][ncl..nch]				*/
/*======================================================================*/
/* m		(o)  matrix to be allocated
   nrl, nrh	(i)  lower and upper row indices
   ncl, nch	(i)  lower and upper column indices
   RESULT	(o)  error code: 0 = no error
				 1 = memory allocation failed		*/
/************************************************************************/
int create_dmatrix(double ***m, int nrl, int nrh, int ncl, int nch) {
  int i,
      nrow = nrh - nrl + 1,
      ncol = nch - ncl + 1;

  /* allocate pointers to rows */
  *m = (double **) malloc((size_t) (nrow*sizeof(double)));
  if (*m == NULL) return (1);
  *m -= nrl;

  /* allocate rows and set pointers to them */
  *(*m + nrl) = (double *) malloc((size_t) (nrow*ncol*sizeof(double)));
  if (*(*m + nrl) == NULL) return (1);
  *(*m + nrl) -= ncl;
  for (i = nrl + 1; i <= nrh; i++) *(*m + i) = *(*m + i-1) + ncol;
  return (0);
}

/*************************************************************29.03.1997*/
/* free_dmatrix: deallocate a double matrix allocated by dmatrix	*/
/*======================================================================*/
/* m		(o)  matrix to be deallocated
   nrl		(i)  lower row index
   ncl		(i)  lower column index					*/
/************************************************************************/
void free_dmatrix(double **m, int nrl, int ncl) {
  free((char *) (m[nrl] + ncl));
  free((char *) (m + nrl));
}

/*************************************************************31.08.2001*/
/* create_imatrix: allocate an int matrix with user-defined subscript	*/
/*		   range: m[nrl..nrh][ncl..nch]				*/
/*======================================================================*/
/* m		(o)  matrix to be allocated
   nrl, nrh	(i)  lower and upper row indices
   ncl, nch	(i)  lower and upper column indices
   RESULT	(o)  error code: 0 = no error
				 1 = memory allocation failed		*/
/************************************************************************/
int create_imatrix(int ***m, int nrl, int nrh, int ncl, int nch) {
  int i,
      nrow = nrh - nrl + 1,
      ncol = nch - ncl + 1;

  /* allocate pointers to rows */
  *m = (int **) malloc((size_t) (nrow*sizeof(int)));
  if (*m == NULL) return (1);
  *m -= nrl;

  /* allocate rows and set pointers to them */
  *(*m + nrl) = (int *) malloc((size_t) (nrow*ncol*sizeof(int)));
  if (*(*m + nrl) == NULL) return (1);
  *(*m + nrl) -= ncl;
  for (i = nrl + 1; i <= nrh; i++) *(*m + i) = *(*m + i-1) + ncol;
  return (0);
}

/*************************************************************06.11.2000*/
/* free_imatrix: deallocate an int matrix allocated by imatrix		*/
/*======================================================================*/
/* m		(o)  matrix to be deallocated
   nrl		(i)  lower row index
   ncl		(i)  lower column index					*/
/************************************************************************/
void free_imatrix(int **m, int nrl, int ncl) {
  free((char *) (m[nrl] + ncl));
  free((char *) (m + nrl));
}

/*************************************************************31.08.2001*/
/* create_dvector: allocate a double vector with user-defined subscript	*/
/*	    range: v[nl..nh]						*/
/*======================================================================*/
/* v		(o)  vector to be allocated
   nl, nh	(i)  lower and upper indices
   RESULT	(o)  error code: 0 = no error
				 1 = memory allocation failed		*/
/************************************************************************/
int create_dvector(double **v, int nl, int nh) {
  *v = (double *) malloc((size_t) ((nh -nl + 1) * sizeof(double)));
  if (*v == NULL) return (1);
  *v -= nl;
  return (0);
}

/*************************************************************29.03.1997*/
/* free_dvector: deallocate a double vector allocated by dvector	*/
/*======================================================================*/
/* v		(i)  vector to be deallocated
   nl		(i)  lower index					*/
/************************************************************************/
void free_dvector(double *v, int nl) {
  free((char *) (v + nl));
}

/*************************************************************31.08.2001*/
/* create_ivector: allocate an int vector with user-defined subscript	*/
/*	    range: v[nl..nh]						*/
/*======================================================================*/
/* v		(o)  vector to be allocated
   nl, nh	(i)  lower and upper indices
   RESULT	(o)  error code: 0 = no error
				 1 = memory allocation failed		*/
/************************************************************************/
int create_ivector(int **v, int nl, int nh) {
  *v = (int *) malloc((size_t) ((nh - nl + 1) * sizeof(int)));
  if (*v == NULL) return (1);
  *v -= nl;
  return (0);
}

/*************************************************************29.03.1997*/
/* free_ivector: deallocate an integer vector allocated by ivector	*/
/*======================================================================*/
/* v		(i)  vector to be deallocated
   nl		(i)  lower index					*/	
/************************************************************************/
void free_ivector(int *v, int nl) {
  free((char *) (v + nl));
}
