/*************************************************************14.08.1998*/
/* ran2: random number generator with reduced sequential correlation	*/
/*									*/
/* This routine uses a congruential algorithm from the manufacturer's	*/
/* C-library and destroys sequential correlation by means of the	*/
/* shuffling table method.						*/
/* Ref.: W.H. Press, B.P. Flannery, S.A. Teukolsky, W.T. Vetterling,	*/
/*	 "Numerical Recipes", Cambridge University Press, Cambridge	*/
/*	 1986, section 7.1						*/
/*									*/
/* congruential random number generator: x[i+1] = (a*x[i]+c) mod m	*/
/* HP implementation: a = 0x5deece66d, c = 11, m = 2^48			*/
/* "Recipes" values:  a = 150889, c = 1366, m = 714025			*/
/************************************************************************/
#include <time.h>
#include <stdlib.h>			/* for drand48() */
#include <math.h>
#include <mathc.h>

double ran2(void) {
  int i;
  static int linit = 0;
  static double r[97], y;

  if (linit == 0) {			/* initialize subroutine */
    linit = 1;
    i = time(NULL);			/* get seconds after 1900 */
    if (i % 2 == 0) i--;
    if (i % 3 == 0) i -= 2;
    srand48(i);				/* seed internal function */
    for (i = 0; i <= 96; i++) r[i] = drand48();
    y = drand48();
  }
  /* calculation of a random number without sequential correlation:
     use previous random number to select a table entry */
  i = 96 * y;
  y = r[i];
  r[i] = drand48();
  return (y);
}
