/*************************************************************24.01.2003*/
/* ran_cgr48: random number generator (congruential method) with reduced*/
/*	      sequential correlation					*/
/*									*/
/* This routine uses a congruential algorithm developed by Hewlett-	*/
/* Packard (drand48(3C)), which randomizes 48 bit instead of the usual	*/
/* 32, and destroys sequential correlation by means of the shuffling	*/
/* table method.							*/
/* Ref.: W.H. Press, B.P. Flannery, S.A. Teukolsky, W.T. Vetterling,	*/
/*	 "Numerical Recipes", Cambridge University Press, Cambridge	*/
/*	 1986, section 7.1						*/
/*									*/
/* congruential random number generator: x[i+1] = (a*x[i]+c) mod m	*/
/* HP implementation: a = 0x5deece66d, c = 11, m = 2^48			*/
/* "Recipes" values:  a = 150889, c = 1366, m = 714025			*/
/*									*/
/* compiling: needs option -Ae under HP-UX				*/
/************************************************************************/
#include <time.h>
#include <math.h>
#include <mathc.h>

#define CGR_A	0x5deece66dLL		/* for congruence algorithm */
#define CGR_C	0xbLL
#define CGR_M	0x1000000000000LL	/* 2^48 */
#define CGR_F	(1.0/((double) CGR_M))
#define NS	97			/* length of shuffling table */


static double drand(void);


unsigned long long randint;


double ran_cgr48(void) {
  int i;
  static int init = 0;
  static double r[NS], y;

  if (init == 0) {			/* initialize subroutine */
    init = 1;
    randint = time(NULL);		/* get seconds after 1900 */
    if (randint % 2 == 0) randint--;
    if (randint % 3 == 0) randint -= 2;
    for (i = 0; i < NS; i++) r[i] = drand();
    y = drand();
  }
  /* calculation of a random number without sequential correlation:
     use previous random number to select a table entry */
  i = (NS - 1) * y;
  y = r[i];
  r[i] = drand();
  return (y);
}

/************************************************************************/
/* drand: auxiliary random number generator (congruential method)	*/
/************************************************************************/
/*======================================================================*/
static double drand(void) {
  randint = (randint * CGR_A + CGR_C) >> 16;
  return (CGR_F * randint);
}
