/*************************************************************10.10.1994*/
/* track: generate initial values for iterative procedures by extra-	*/
/*	  polation from previous results (in other words, store arrays	*/
/*	  of results, y[], for various values of the control variable	*/
/*	  x, then extrapolate for a new x value)			*/
/*									*/
/* Track stores up to VMAX previous vectors. If more vectors are stored,*/
/* the oldest entry is overwritten.					*/
/*======================================================================*/
/* x		(i)  independent variable
   n		(i)  number of dependent variables
   y[n]		(io) array of dependent variables
   mode		(i)  mode control:
   		       0 = reset
		       1 = store an array
		       2 = extrapolate
   restr[n]	(i)  restriction of array values:
		       0 = no restriction
		       1 = y values positive
		       2 = y values < 1
		       3 = y between 0 and 1				*/
/************************************************************************/
#include <math.h>
#include <mathc.h>

#define VMAX		3		/* number of entries stored */

void track(double x, int n, double *y, int mode, int *restr) {
  int i, j;
  double h, d12, d13, d23;
  static int k = 0,
	     ninit = 0;
  static double xtbl[VMAX];
  static dmatrix ytbl;

  if (ninit != n) {			/* initialize arrays */
    dmatrix_free(&ytbl);
    dmatrix_create(&ytbl, 0, 2, 0, n - 1);
    k = 0;
    ninit = n;
  }

  switch (mode) {
  case 0:				/* purge table */
    k = 0;
    break;
  case 1:				/* enter a new vector */
    if (k == VMAX) {
      for (i = 1; i < VMAX; i++) {
	xtbl[i - 1] = xtbl[i];
	for (j = 0; j < n; j++) ytbl.a[i - 1][j] = ytbl.a[i][j];
      }
    }
    if (k < VMAX) k++;
    xtbl[k - 1] = x;
    for (j = 0; j < n; j++) {
      h = (restr[j] & 2) ? ((y[j] < 1.0) ? -log(1.0 - y[j]) : DHIGH) : y[j];
      if (restr[j] & 1) h = (h > 0.0) ? log(h) : -DHIGH;
      ytbl.a[k - 1][j] = h;
    }
    break;
  case 2:				/* extrapolate */
    if (k >= 1 ) {
      for (j = 0; j < n; j++) y[j] = ytbl.a[k - 1][j];
    }
    if (k >= 2) {			/* ... linear */
      d12 = xtbl[k - 1] - xtbl[k - 2];
      h = (x - xtbl[k - 1]) / d12;
      for (j = 0; j < n; j++) y[j] += (ytbl.a[k - 1][j] - ytbl.a[k - 2][j]) * h;
    }
    if (k >= 3) {			/* ... quadratic */
      d23 = xtbl[k - 2] - xtbl[k - 3];
      d13 = xtbl[k - 1] - xtbl[k - 3];
      h = -(x - xtbl[k - 1]) * (x - xtbl[k - 2])/(d13 * d12 * d23);
      for (j = 0; j < n; j++)
	y[j] += (-ytbl.a[k - 3][j] * d12 + ytbl.a[k - 2][j] * d13
	- ytbl.a[k - 1][j] * d23) * h;
    }
    for (j = 0; j < n; j++) {
      if (restr[j] % 2 == 1) y[j] = exp(y[j]);
      if (restr[j] >= 2) y[j] = 1.0 - exp(-y[j]);
    }
  }
}
