/************************************************************************/
/* test1: test the performance of parameter fitting routines		*/
/*									*/
/* The program can be used for fit_marquardt, fit_powell, and		*/
/* fit_simplex; change the preprocessor statements below as required.	*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin sequence:
   1. 3 initial values for amplitude, center, and width of Gaussian
   2. 5 data pairs							*/
/************************************************************************/
#include <stdio.h>
#include <math.h>
#include <mathc.h>

#define MARQUARDT	0
#define POWELL		0
#define SIMPLEX		1

struct exp_data {
  int n;
  double t[5], u[5];
};


int f(double *x, double *y, struct exp_data *a);/* prototype of object func. */


int main(void) {
  int i;
  double x[3], dx[3], ynorm, y[5];
  struct exp_data d;

  for (i = 0; i < 3; i++) scanf("%lf", &x[i]);
  for (i = 0; i < 5; i++) scanf("%lf%lf", &d.t[i], &d.u[i]);
  d.n = 5;
  i = f(x, y, &d);
  printf("before:\n");
  for (i = 0; i < 5; i++) printf("%d %9.6lf %9.6lf %17.8e\n", i,
  d.t[i], d.u[i], y[i]);
#if MARQUARDT
  i = fit_marquardt(f, 3, x, 5, &d, 50, 1, &ynorm);
#elif POWELL
  i = fit_powell(f, 3, x, 5, &d, 50, 1, &ynorm);
#else
  dx[0] = dx[1] = dx[2] = 1.0;
  i = fit_simplex(f, 3, x, dx, 5, &d, 50, 1, &ynorm);
#endif
  printf("\nerror code: %d   rms: %e\n", i, ynorm);
  for (i = 0; i < 3; i++) printf("%lf ", x[i]);
  printf("\n");
  i = f(x, y, &d);
  for (i = 0; i < 5; i++) printf("%d %9.6lf %9.6lf %17.8e\n",
  i, d.t[i], d.u[i], y[i]);
  return (MTH_GOOD);
}

int f(double *x, double *y, struct exp_data *a) {
 int i;
 for (i = 0; i < 5; i++) y[i] = x[0] * exp(-SQ((a->t[i] - x[1]) / x[2]))
 - a->u[i];
 return (MTH_GOOD);
}
