/************************************************************************/
/* test1: test solver for ordinary differential equations		*/
/*	  ode_runge_kutta()						*/
/* (solve y' + y = 0, with y(0) = 1)					*/
/*======================================================================*/
/* I/O channels:
   stdin
   stdout

   stdin sequence:
   1. Enter integration limit xmax and the tabulation step dx.		*/
/************************************************************************/
#include <stdio.h>
#include <math.h>
#include <mathc.h>


double f(double x, double y, void *s);


int main(void) {
  int i;
  double xmax, dx, x0, y0, y;

  scanf("%lf%lf", &xmax, &dx);
  if (xmax * dx < 0.0) {
    printf("xmax and dx must have equal signs.\n");
    return (MTH_FAIL);
  }

  x0 = 0.0;
  y0 = 1.0;
  printf("          x           y     exp(-x)\n");
  printf("%11.4e %11.4e %11.4e\n", x0, y0, 1.0);

  while ((x0 - xmax) * dx < 0.0) {
    ode_runge_kutta(f, NULL, x0, y0, x0 + dx, &y);
    x0 += dx;
    y0 = y;
    printf("%11.4e %11.4e %11.4e\n", x0, y0, exp(-x0));
  }
  return (MTH_GOOD);
}

/************************************************************************/
/* ODE function: y' = f(x,y)						*/
/*======================================================================*/
/* x, y		(i)  arguments
   s		(i)  pass-through argument
   RESULT	(o)  -y							*/
/************************************************************************/
double f(double x, double y, void *s) {
  return (-y);
}
